@@ -16,6 +16,12 @@ with NML; if not, write to the Free Software Foundation, Inc.,
 from nml import generic
 from nml.actions import base_action, real_sprite
 
+"""
+Maximum number of sprites per block.
+This can be increased by switching to extended Action1.
+"""
+max_sprite_block_size = 0xFF
+
 class Action1(base_action.BaseAction):
     """
     Class representing an Action1
@@ -88,14 +94,14 @@ class SpritesetCollection(base_action.BaseAction):
         @return: True iff the given spritesets can be added to this collection.
         @rtype: C{bool}
         """
-        assert len(spritesets) < 0x100
+        assert len(spritesets) <= max_sprite_block_size
         if feature != self.feature:
             return False
         for spriteset in spritesets:
-            if len(real_sprite.parse_sprite_list(spriteset.sprite_list, spriteset.pcx)) != self.num_sprites_per_spriteset:
+            if len(real_sprite.parse_sprite_data(spriteset)) != self.num_sprites_per_spriteset:
                 return False
-        num_new_sets = len([x for x in spritesets if (x not in self.spritesets)])
-        return len(self.spritesets) + num_new_sets < 0x100
+        num_new_sets = sum(1 for x in spritesets if x not in self.spritesets)
+        return len(self.spritesets) + num_new_sets <= max_sprite_block_size
 
     def add(self, spritesets):
         """
@@ -134,12 +140,28 @@ class SpritesetCollection(base_action.BaseAction):
         """
         actions = [Action1(self.feature, len(self.spritesets), self.num_sprites_per_spriteset)]
         for idx in range(len(self.spritesets)):
-            for spriteset, spriteset_offset in self.spritesets.iteritems():
+            for spriteset, spriteset_offset in list(self.spritesets.items()):
                 if idx == spriteset_offset:
-                    actions.extend(real_sprite.parse_sprite_list(spriteset.sprite_list, spriteset.pcx, block_name = spriteset.name))
+                    actions.extend(real_sprite.parse_sprite_data(spriteset))
                     break
         return actions
-        
+
+"""
+Statistics about spritesets.
+The 1st field of type C{int} contains the largest block of consecutive spritesets.
+The 2nd field of type L{Position} contains a positional reference to the largest block of consecutive spritesets.
+"""
+spriteset_stats = (0, None)
+
+def print_stats():
+    """
+    Print statistics about used ids.
+    """
+    if spriteset_stats[0] > 0:
+        # NML uses as many concurrent spritesets as possible to prevent sprite duplication.
+        # So, instead of the actual amount, we rather print the biggest unsplittable block, since that is what matters.
+        generic.print_info("Concurrent spritesets: {}/{} ({})".format(spriteset_stats[0], max_sprite_block_size, str(spriteset_stats[1])))
+
 """
 The collection which was previoulsy used. add_to_action1 will try to reuse this
 collection as long as possible to reduce the duplication of sprites. As soon
@@ -159,21 +181,28 @@ def add_to_action1(spritesets, feature, pos):
     @param feature: Feature of the spritesets.
     @type feature: C{int}
 
+    @param pos: Position reference to source.
+    @type  pos: L{Position}
+
     @return: List of collections that needs to be added to the global action list.
     @rtype: C{list} of L{SpritesetCollection}.
     """
     if not spritesets:
         return []
 
-    setsize = len(real_sprite.parse_sprite_list(spritesets[0].sprite_list, spritesets[0].pcx))
+    setsize = len(real_sprite.parse_sprite_data(spritesets[0]))
     for spriteset in spritesets:
-        if setsize != len(real_sprite.parse_sprite_list(spriteset.sprite_list, spriteset.pcx)):
+        if setsize != len(real_sprite.parse_sprite_data(spriteset)):
             raise generic.ScriptError("Using spritesets with different sizes in a single sprite group / layout is not possible", pos)
 
+    global spriteset_stats
+    if spriteset_stats[0] < len(spritesets):
+        spriteset_stats = (len(spritesets), pos)
+
     global last_spriteset_collection
     actions = []
     if last_spriteset_collection is None or not last_spriteset_collection.can_add(spritesets, feature):
-        last_spriteset_collection = SpritesetCollection(feature, len(real_sprite.parse_sprite_list(spritesets[0].sprite_list, spritesets[0].pcx)))
+        last_spriteset_collection = SpritesetCollection(feature, len(real_sprite.parse_sprite_data(spritesets[0])))
         actions.append(last_spriteset_collection)
 
     last_spriteset_collection.add(spritesets)
@@ -195,3 +224,24 @@ def get_action1_index(spriteset):
     """
     assert last_spriteset_collection is not None
     return last_spriteset_collection.get_index(spriteset)
+
+def make_cb_failure_action1(feature):
+    """
+    Create an action1 that may be used for a callback failure
+    If the last action1 is of the correct feature, no new action1 is needed
+    Else, add a new action1 with 1 spriteset containing 0 sprites
+
+    @param feature: Feature of the requested action 1
+    @type feature: C{int}
+
+    @return: List of actions to append (if any) and action1 index to use
+    @rtype: C{tuple} of (C{list} of L{BaseAction}, C{int})
+    """
+    global last_spriteset_collection
+    if last_spriteset_collection is not None and last_spriteset_collection.feature == feature:
+        actions = []
+    else:
+        last_spriteset_collection = None
+        actions = [Action1(feature, 1, 0)]
+    return (actions, 0) # Index is currently always 0, but will change with ext. A1
+
