@@ -13,13 +13,13 @@ You should have received a copy of the GNU General Public License along
 with NML; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA."""
 
-from nml import generic, global_constants, expression, nmlop
+from nml import generic, expression, nmlop
 from nml.actions import action2, action6, actionD, action1, action2var, real_sprite
 from nml.ast import general
 
 class Action2Layout(action2.Action2):
-    def __init__(self, feature, name, ground_sprite, sprite_list, param_registers):
-        action2.Action2.__init__(self, feature, name)
+    def __init__(self, feature, name, pos, ground_sprite, sprite_list, param_registers):
+        action2.Action2.__init__(self, feature, name, pos)
         assert ground_sprite.type == Action2LayoutSpriteType.GROUND
         self.ground_sprite = ground_sprite
         self.sprite_list = sprite_list
@@ -27,6 +27,10 @@ class Action2Layout(action2.Action2):
 
     def resolve_tmp_storage(self):
         for reg in self.param_registers:
+            if not self.tmp_locations:
+                raise generic.ScriptError("There are not enough registers available " +
+                        "to perform all required computations in switch blocks. " +
+                        "Please reduce the complexity of your code.", self.pos)
             location = self.tmp_locations[0]
             self.remove_tmp_location(location, False)
             reg.set_register(location)
@@ -245,9 +249,9 @@ class Action2LayoutSprite(object):
         name = name.value
 
         if not name in self.params:
-            raise generic.ScriptError("Unknown sprite parameter '%s'" % name, value.pos)
+            raise generic.ScriptError("Unknown sprite parameter '{}'".format(name), value.pos)
         if self.is_set(name):
-            raise generic.ScriptError("Sprite parameter '%s' can be set only once per sprite." % name, value.pos)
+            raise generic.ScriptError("Sprite parameter '{}' can be set only once per sprite.".format(name), value.pos)
 
         self.params[name]['value'] = self.params[name]['validator'](name, value)
         self.params[name]['is_set'] = True
@@ -270,7 +274,7 @@ class Action2LayoutSprite(object):
             id_dicts = [(spriteset.labels, lambda val, pos: expression.ConstantNumeric(val, pos))]
             offset = action2var.reduce_varaction2_expr(sg_ref.param_list[0], self.feature, self.extra_dicts + id_dicts)
             if isinstance(offset, expression.ConstantNumeric):
-                generic.check_range(offset.value, 0, len(real_sprite.parse_sprite_list(spriteset.sprite_list, spriteset.pcx)) - 1, "offset within spriteset", sg_ref.pos)
+                generic.check_range(offset.value, 0, len(real_sprite.parse_sprite_data(spriteset)) - 1, "offset within spriteset", sg_ref.pos)
         else:
             raise generic.ScriptError("Expected 0 or 1 parameter, got " + str(len(sg_ref.param_list)), sg_ref.pos)
 
@@ -343,7 +347,7 @@ class Action2LayoutSprite(object):
             else:
                 assert name in ('xextent', 'yextent', 'zextent')
                 if not isinstance(value, expression.ConstantNumeric):
-                    raise generic.ScriptError("Value of '%s' must be a compile-time constant number." % name, value.pos)
+                    raise generic.ScriptError("Value of '{}' must be a compile-time constant number.".format(name), value.pos)
                 generic.check_range(value.value, 0, 255, name, value.pos)
                 return value
         # Value must be written to a register
@@ -357,16 +361,25 @@ class Action2LayoutSprite(object):
         return expression.ConstantNumeric(0)
 
     def _validate_hide_sprite(self, name, value):
-        self.create_register(name, expression.Not(value))
+        value = expression.Not(value)
+        try:
+            value = value.reduce()
+        except:
+            pass
+        self.create_register(name, value)
         return None
 
-def get_layout_action2s(spritelayout, feature):
+def get_layout_action2s(spritelayout, feature, spr_pos):
+    """
+    @param spr_pos: Position information of the sprite view.
+    @type  spr_pos: L{Position}
+    """
     ground_sprite = None
     building_sprites = []
     actions = []
 
     if feature not in action2.features_sprite_layout:
-        raise generic.ScriptError("Sprite layouts are not supported for feature '%s'." % general.feature_name(feature))
+        raise generic.ScriptError("Sprite layouts are not supported for feature '{}'.".format(general.feature_name(feature)))
 
     # Allocate registers
     param_map = {}
@@ -378,7 +391,7 @@ def get_layout_action2s(spritelayout, feature):
     param_map = (param_map, lambda value, pos: action2var.VarAction2LoadLayoutParam(value))
     spritelayout.register_map[feature] = param_registers
 
-    # Reduce all expresssions, can't do that earlier as feature is not known
+    # Reduce all expressions, can't do that earlier as feature is not known
     all_sprite_sets = []
     layout_sprite_list = [] # Create a new structure
     for layout_sprite in spritelayout.layout_sprite_list:
@@ -397,7 +410,7 @@ def get_layout_action2s(spritelayout, feature):
     temp_registers = []
     for type, pos, param_list in layout_sprite_list:
         if type.value not in layout_sprite_types:
-            raise generic.ScriptError("Invalid sprite type '%s' encountered. Expected 'ground', 'building', or 'childsprite'." % type.value, type.pos)
+            raise generic.ScriptError("Invalid sprite type '{}' encountered. Expected 'ground', 'building', or 'childsprite'.".format(type.value), type.pos)
         sprite = Action2LayoutSprite(feature, layout_sprite_types[type.value], pos, [param_map])
         for name, value in param_list:
             sprite.set_param(name, value)
@@ -412,7 +425,7 @@ def get_layout_action2s(spritelayout, feature):
     if ground_sprite is None:
         if len(building_sprites) == 0:
             #no sprites defined at all, that's not very much.
-            raise generic.ScriptError("Sprite layout requires at least one sprite", spritegroup.pos)
+            raise generic.ScriptError("Sprite layout requires at least one sprite", spr_pos)
         #set to 0 for no ground sprite
         ground_sprite = Action2LayoutSprite(feature, Action2LayoutSpriteType.GROUND)
         ground_sprite.set_param(expression.Identifier('sprite'), expression.ConstantNumeric(0))
@@ -422,26 +435,19 @@ def get_layout_action2s(spritelayout, feature):
 
     offset = 4
     sprite_num = ground_sprite.get_sprite_number()
-    if not isinstance(sprite_num, expression.ConstantNumeric):
-        param, extra_actions = actionD.get_tmp_parameter(sprite_num)
-        actions.extend(extra_actions)
-        act6.modify_bytes(param, 4, offset)
-    offset += 4
+    sprite_num, offset = actionD.write_action_value(sprite_num, actions, act6, offset, 4)
     offset += ground_sprite.get_registers_size()
 
     for sprite in building_sprites:
         sprite_num = sprite.get_sprite_number()
-        if not isinstance(sprite_num, expression.ConstantNumeric):
-            param, extra_actions = actionD.get_tmp_parameter(sprite_num)
-            actions.extend(extra_actions)
-            act6.modify_bytes(param, 4, offset)
+        sprite_num, offset = actionD.write_action_value(sprite_num, actions, act6, offset, 4)
         offset += sprite.get_registers_size()
-        offset += 7 if sprite.type == Action2LayoutSpriteType.CHILD else 10
+        offset += 3 if sprite.type == Action2LayoutSpriteType.CHILD else 6
 
     if len(act6.modifications) > 0:
         actions.append(act6)
 
-    layout_action = Action2Layout(feature, spritelayout.name.value + (" - feature %02X" % feature), ground_sprite, building_sprites, param_registers)
+    layout_action = Action2Layout(feature, spritelayout.name.value + " - feature {:02X}".format(feature), spritelayout.pos, ground_sprite, building_sprites, param_registers)
     actions.append(layout_action)
 
     if temp_registers:
@@ -467,7 +473,7 @@ def get_layout_action2s(spritelayout, feature):
             extra_act6.modify_bytes(mod.param, mod.size, mod.offset + 4)
         if len(extra_act6.modifications) > 0: actions.append(extra_act6)
 
-        varaction2 = action2var.Action2Var(feature, "%s@registers - feature %02X" % (spritelayout.name.value, feature), 0x89)
+        varaction2 = action2var.Action2Var(feature, "{}@registers - feature {:02X}".format(spritelayout.name.value, feature), spritelayout.pos, 0x89)
         varaction2.var_list = varact2parser.var_list
         ref = expression.SpriteGroupRef(spritelayout.name, [], None, layout_action)
         varaction2.ranges.append(action2var.VarAction2Range(expression.ConstantNumeric(0), expression.ConstantNumeric(0), ref, ''))
@@ -485,3 +491,22 @@ def get_layout_action2s(spritelayout, feature):
 
     action6.free_parameters.restore()
     return actions
+
+def make_empty_layout_action2(feature, pos):
+    """
+    Make an empty layout action2
+    For use with failed callbacks
+
+    @param feature: Feature of the sprite layout to create
+    @type feature: C{int}
+
+    @param pos: Positional context.
+    @type  pos: L{Position}
+
+    @return: The created sprite layout action2
+    @rtype: L{Action2Layout}
+    """
+    ground_sprite = Action2LayoutSprite(feature, Action2LayoutSpriteType.GROUND)
+    ground_sprite.set_param(expression.Identifier('sprite'), expression.ConstantNumeric(0))
+    return Action2Layout(feature, "@CB_FAILED_LAYOUT{:02X}".format(feature), pos, ground_sprite, [], [])
+
