@@ -32,8 +32,8 @@ class Action2Production(action2.Action2):
     @ivar again: Number (v0) or register (v1), production CB will be run again if nonzero.
     @type again C{int} or L{VarAction2Var}
     """
-    def __init__(self, name, version, sub_in, add_out, again):
-        action2.Action2.__init__(self, 0x0A, name)
+    def __init__(self, name, pos, version, sub_in, add_out, again):
+        action2.Action2.__init__(self, 0x0A, name, pos)
         self.version = version
         assert version == 0 or version == 1
         self.sub_in = sub_in
@@ -42,8 +42,8 @@ class Action2Production(action2.Action2):
         assert len(self.add_out) == 2
         self.again = again
 
-    def prepare_output(self):
-        action2.Action2.prepare_output(self)
+    def prepare_output(self, sprite_num):
+        action2.Action2.prepare_output(self, sprite_num)
 
     def write(self, file):
         cargo_size = 2 if self.version == 0 else 1
@@ -73,19 +73,12 @@ def get_production_actions(produce):
 
     result_list = []
     varact2parser = action2var.Varaction2Parser(0x0A)
-    if all(map(lambda x: x.supported_by_actionD(False), produce.param_list)):
+    if all(x.supported_by_actionD(False) for x in produce.param_list):
         version = 0
+        offset = 4
         for i, param in enumerate(produce.param_list):
-            if isinstance(param, expression.ConstantNumeric):
-                result_list.append(param.value)
-            else:
-                if isinstance(param, expression.Parameter) and isinstance(param.num, expression.ConstantNumeric):
-                    param_num = param.num.value
-                else:
-                    param_num, tmp_param_actions = actionD.get_tmp_parameter(param)
-                    action_list.extend(tmp_param_actions)
-                act6.modify_bytes(param_num, 2 if i < 5 else 1, 1 + 2 * i)
-                result_list.append(0)
+            result, offset = actionD.write_action_value(param, action_list, act6, offset, 2 if i < 5 else 1)
+            result_list.append(result.value)
     else:
         version = 1
         for i, param in enumerate(produce.param_list):
@@ -105,14 +98,14 @@ def get_production_actions(produce):
                 varact2parser.var_list_size += store_tmp.get_size() + 1 # Add 1 for operator
 
     if len(act6.modifications) > 0: action_list.append(act6)
-    prod_action = Action2Production(produce.name.value, version, result_list[0:3], result_list[3:5], result_list[5])
+    prod_action = Action2Production(produce.name.value, produce.pos, version, result_list[0:3], result_list[3:5], result_list[5])
     action_list.append(prod_action)
 
     if len(varact2parser.var_list) == 0:
         produce.set_action2(prod_action, 0x0A)
     else:
         # Create intermediate varaction2
-        varaction2 = action2var.Action2Var(0x0A, '%s@registers' % produce.name.value, 0x89)
+        varaction2 = action2var.Action2Var(0x0A, '{}@registers'.format(produce.name.value), produce.pos, 0x89)
         varaction2.var_list = varact2parser.var_list
         action_list.extend(varact2parser.extra_actions)
         extra_act6 = action6.Action6()
@@ -133,3 +126,17 @@ def get_production_actions(produce):
     action6.free_parameters.restore()
 
     return action_list
+
+def make_empty_production_action2(pos):
+    """
+    Make an empty production action2
+    For use with failed callbacks
+
+    @param pos: Positional context.
+    @type  pos: L{Position}
+
+    @return: The created production action2
+    @rtype: L{Action2Production}
+    """
+    return Action2Production("@CB_FAILED_PROD", pos, 0, [0, 0, 0], [0, 0], 0)
+
