@@ -14,11 +14,11 @@ with NML; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA."""
 
 from nml.actions import action2, action2var, action2real, action6
-from nml import generic, expression, global_constants, nmlop
+from nml import generic, expression, nmlop
 
 class Action2Random(action2.Action2):
-    def __init__(self, feature, name, type_byte, count, triggers, randbit, nrand):
-        action2.Action2.__init__(self, feature, name)
+    def __init__(self, feature, name, pos, type_byte, count, triggers, randbit, nrand):
+        action2.Action2.__init__(self, feature, name, pos)
         self.type_byte = type_byte
         self.count = count
         self.triggers = triggers
@@ -26,8 +26,8 @@ class Action2Random(action2.Action2):
         self.nrand = nrand
         self.choices = []
 
-    def prepare_output(self):
-        action2.Action2.prepare_output(self)
+    def prepare_output(self, sprite_num):
+        action2.Action2.prepare_output(self, sprite_num)
         for choice in self.choices:
             if isinstance(choice.result, expression.SpriteGroupRef):
                 choice.result = choice.result.get_action2_id(self.feature)
@@ -106,23 +106,23 @@ def parse_randomswitch_type(random_switch):
     # Extract some stuff we'll often need
     type_str = random_switch.type.value
     type_pos = random_switch.type.pos
-    feature_val = random_switch.feature_set.copy().pop()
+    feature_val = next(iter(random_switch.feature_set))
 
     # Validate type name / param combination
     if type_str not in random_types[feature_val]:
-        raise generic.ScriptError("Invalid combination for random_switch feature %d and type '%s'. " % (feature_val, type_str), type_pos)
+        raise generic.ScriptError("Invalid combination for random_switch feature {:d} and type '{}'. ".format(feature_val, type_str), type_pos)
     type_info = random_types[feature_val][type_str]
 
     count_expr = None
     if random_switch.type_count is None:
         # No param given
         if type_info['param'] == 1:
-            raise generic.ScriptError("Value '%s' for random_switch parameter 2 'type' requires a parameter." % type_str, type_pos)
+            raise generic.ScriptError("Value '{}' for random_switch parameter 2 'type' requires a parameter.".format(type_str), type_pos)
         count = None
     else:
         # Param given
         if type_info['param'] == 0:
-            raise generic.ScriptError("Value '%s' for random_switch parameter 2 'type' should not have a parameter." % type_str, type_pos)
+            raise generic.ScriptError("Value '{}' for random_switch parameter 2 'type' should not have a parameter.".format(type_str), type_pos)
         if isinstance(random_switch.type_count, expression.ConstantNumeric) and 1 <= random_switch.type_count.value <= 15:
             count = random_switch.type_count.value
         else:
@@ -131,7 +131,7 @@ def parse_randomswitch_type(random_switch):
         count = type_info['value'] | count
 
     if random_switch.triggers.value != 0 and not type_info['triggers']:
-        raise generic.ScriptError("Triggers may not be set for random_switch feature %d and type '%s'. " % (feature_val, type_str), type_pos)
+        raise generic.ScriptError("Triggers may not be set for random_switch feature {:d} and type '{}'. ".format(feature_val, type_str), type_pos)
 
     # Determine type byte and random bits
     type_byte = type_info['type']
@@ -183,19 +183,24 @@ def parse_randomswitch_dependencies(random_switch, start_bit, bits_available, nr
         if act2 is None: continue # May happen if said random switch is not used and therefore not parsed
         if act2_to_copy is not None:
             if act2_to_copy.randbit != act2.randbit:
-                raise generic.ScriptError("random_switch '%s' cannot be dependent on both '%s' and '%s' as these are independent of eachother." %
-                    (random_switch.name.value, act2_to_copy.name, act2.name), random_switch.pos)
+                msg = "random_switch '{}' cannot be dependent on both '{}' and '{}' as these are independent of each other."
+                msg = msg.format(random_switch.name.value, act2_to_copy.name, act2.name)
+                raise generic.ScriptError(msg, random_switch.pos)
+
             if act2_to_copy.nrand != act2.nrand:
-                raise generic.ScriptError("random_switch '%s' cannot be dependent on both '%s' and '%s' as they don't use the same amount of random data." %
-                    (random_switch.name.value, act2_to_copy.name, act2.name), random_switch.pos)
+                msg = "random_switch '{}' cannot be dependent on both '{}' and '{}' as they don't use the same amount of random data."
+                msg = msg.format(random_switch.name.value, act2_to_copy.name, act2.name)
+                raise generic.ScriptError(msg, random_switch.pos)
         else:
             act2_to_copy = act2
 
     if act2_to_copy is not None:
         randbit = act2_to_copy.randbit
         if nrand > act2_to_copy.nrand:
-            raise generic.ScriptError("random_switch '%s' cannot be dependent on '%s' as it requires more random data." %
-                (random_switch.name.value, act2_to_copy.name), random_switch.pos)
+            msg = "random_switch '{}' cannot be dependent on '{}' as it requires more random data."
+            msg = msg.format(random_switch.name.value, act2_to_copy.name)
+            raise generic.ScriptError(msg, random_switch.pos)
+
         nrand = act2_to_copy.nrand
     else:
         randbit = -1
@@ -211,7 +216,9 @@ def parse_randomswitch_dependencies(random_switch, start_bit, bits_available, nr
     if randbit != -1:
         #randbit has already been determined. Check that it is suitable
         if possible_mask & (required_mask << randbit) != (required_mask << randbit):
-            raise generic.ScriptError("Combination of dependence on and independence from random_switches is not possible for random_switch '%s'." % random_switch.name.value, random_switch.pos)
+            msg = "Combination of dependence on and independence from random_switches is not possible for random_switch '{}'."
+            msg = msg.format(random_switch.name.value)
+            raise generic.ScriptError(msg, random_switch.pos)
     else:
         #find a suitable randbit
         for i in range(start_bit, bits_available + start_bit):
@@ -219,7 +226,9 @@ def parse_randomswitch_dependencies(random_switch, start_bit, bits_available, nr
                 randbit = i
                 break
         else:
-            raise generic.ScriptError("Independence of all given random_switches is not possible for random_switch '%s'." % random_switch.name.value, random_switch.pos)
+            msg = "Independence of all given random_switches is not possible for random_switch '{}'."
+            msg = msg.format(random_switch.name.value)
+            raise generic.ScriptError(msg, random_switch.pos)
 
     return randbit, nrand
 
@@ -234,7 +243,7 @@ def parse_randomswitch(random_switch):
     @rtype: C{list} of L{BaseAction}
     """
     action_list = action2real.create_spriteset_actions(random_switch)
-    feature = random_switch.feature_set.copy().pop()
+    feature = next(iter(random_switch.feature_set))
     type_byte, count, count_expr, start_bit, bits_available = parse_randomswitch_type(random_switch)
 
     total_prob = sum([choice.probability.value for choice in random_switch.choices])
@@ -244,20 +253,22 @@ def parse_randomswitch(random_switch):
 
     # Verify that enough random data is available
     if min(1 << bits_available, 0x80) < nrand:
-        raise generic.ScriptError("The maximum sum of all random_switch probabilities is %d, encountered %d." % (min(1 << bits_available, 0x80), total_prob), random_switch.pos)
+        msg = "The maximum sum of all random_switch probabilities is {:d}, encountered {:d}."
+        msg = msg.format(min(1 << bits_available, 0x80), total_prob)
+        raise generic.ScriptError(msg, random_switch.pos)
 
     randbit, nrand = parse_randomswitch_dependencies(random_switch, start_bit, bits_available, nrand)
 
-    random_action2 = Action2Random(feature, random_switch.name.value, type_byte, count, random_switch.triggers.value, randbit, nrand)
+    random_action2 = Action2Random(feature, random_switch.name.value, random_switch.pos, type_byte, count, random_switch.triggers.value, randbit, nrand)
     random_switch.random_act2 = random_action2
 
     action6.free_parameters.save()
     act6 = action6.Action6()
-    offset = 8
+    offset = 8 if count is not None else 7
 
     #divide the 'extra' probabilities in an even manner
     i = 0
-    resulting_prob = dict(zip(random_switch.choices, [c.probability.value for c in random_switch.choices]))
+    resulting_prob = dict((c, c.probability.value) for c in random_switch.choices)
     while i < (nrand - total_prob):
         best_choice = None
         best_ratio = 0
@@ -273,9 +284,9 @@ def parse_randomswitch(random_switch):
 
     for choice in random_switch.choices:
         res_prob = resulting_prob[choice]
-        result, comment = action2var.parse_result(choice.result, action_list, act6, offset, random_action2, None, 0x89, res_prob)
+        result, comment = action2var.parse_result(choice.result.value, action_list, act6, offset, random_action2, None, 0x89, res_prob)
         offset += res_prob * 2
-        comment = "(%d/%d) -> (%d/%d): " % (choice.probability.value, total_prob, res_prob, nrand) + comment
+        comment = "({:d}/{:d}) -> ({:d}/{:d}): ".format(choice.probability.value, total_prob, res_prob, nrand) + comment
         random_action2.choices.append(RandomAction2Choice(result, res_prob, comment))
 
     if len(act6.modifications) > 0: action_list.append(act6)
@@ -285,7 +296,7 @@ def parse_randomswitch(random_switch):
         random_switch.set_action2(random_action2, feature)
     else:
         # Create intermediate varaction2
-        varaction2 = action2var.Action2Var(feature, '%s@registers' % random_switch.name.value, 0x89)
+        varaction2 = action2var.Action2Var(feature, '{}@registers'.format(random_switch.name.value), random_switch.pos, 0x89)
         varact2parser = action2var.Varaction2Parser(feature)
         varact2parser.parse_expr(count_expr)
         varaction2.var_list = varact2parser.var_list
