@@ -14,7 +14,7 @@ with NML; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA."""
 
 from nml.actions import action2, action2real, action2var_variables, action4, action6, actionD
-from nml import expression, generic, global_constants, nmlop, unit
+from nml import expression, generic, global_constants, nmlop
 from nml.ast import general
 
 class Action2Var(action2.Action2):
@@ -32,26 +32,30 @@ class Action2Var(action2.Action2):
 
     @ivar ranges: List of return value ranges. Each range contains a minimum and
                   a maximum value and a return value. The list is checked in order,
-                  if the result of the computation is between the miminum and
+                  if the result of the computation is between the minimum and
                   maximum (inclusive) of one range the result of that range is
                   returned. The result can be either an integer of another
                   action2.
     @ivar ranges: C{list} of L{VarAction2Range}
     """
-    def __init__(self, feature, name, type_byte):
-        action2.Action2.__init__(self, feature, name)
+    def __init__(self, feature, name, pos, type_byte):
+        action2.Action2.__init__(self, feature, name, pos)
         self.type_byte = type_byte
         self.ranges = []
 
     def resolve_tmp_storage(self):
         for var in self.var_list:
             if isinstance(var, VarAction2StoreTempVar):
+                if not self.tmp_locations:
+                    raise generic.ScriptError("There are not enough registers available " +
+                            "to perform all required computations in switch blocks. " +
+                            "Please reduce the complexity of your code.", self.pos)
                 location = self.tmp_locations[0]
                 self.remove_tmp_location(location, False)
                 var.set_register(location)
 
-    def prepare_output(self):
-        action2.Action2.prepare_output(self)
+    def prepare_output(self, sprite_num):
+        action2.Action2.prepare_output(self, sprite_num)
         for i in range(0, len(self.var_list) - 1, 2):
             self.var_list[i].shift |= 0x20
 
@@ -294,14 +298,6 @@ class Modification(object):
         self.size = size
         self.offset = offset
 
-def pow2(expr):
-    #2**x = (1 ror (32 - x))
-    if isinstance(expr, expression.ConstantNumeric):
-        return expression.ConstantNumeric(1 << expr.value)
-    expr = expression.BinOp(nmlop.SUB, expression.ConstantNumeric(32), expr)
-    expr = expression.BinOp(nmlop.ROT_RIGHT, expression.ConstantNumeric(1), expr)
-    return expr
-
 class Varaction2Parser(object):
     def __init__(self, feature):
         self.feature = feature # Depends on feature and var_range
@@ -356,22 +352,13 @@ class Varaction2Parser(object):
             expr = expression.BinOp(nmlop.AND, expr, expression.ConstantNumeric(1))
             expr = expression.BinOp(nmlop.XOR, expr, expression.ConstantNumeric(1))
 
-        elif expr.op == nmlop.SHIFT_LEFT:
-            #a << b ==> a * (2**b)
-            expr = expression.BinOp(nmlop.MUL, expr.expr1, pow2(expr.expr2))
-        elif expr.op == nmlop.SHIFT_RIGHT:
-            #a >> b ==> a / (2**b)
-            expr = expression.BinOp(nmlop.DIV, expr.expr1, pow2(expr.expr2))
-        elif expr.op == nmlop.SHIFTU_RIGHT:
-            #a >>> b ==> (uint)a / (2**b)
-            expr = expression.BinOp(nmlop.DIVU, expr.expr1, pow2(expr.expr2))
         elif expr.op == nmlop.HASBIT:
             # hasbit(x, n) ==> (x >> n) & 1
-            expr = expression.BinOp(nmlop.DIV, expr.expr1, pow2(expr.expr2))
+            expr = expression.BinOp(nmlop.SHIFTU_RIGHT, expr.expr1, expr.expr2)
             expr = expression.BinOp(nmlop.AND, expr, expression.ConstantNumeric(1))
         elif expr.op == nmlop.NOTHASBIT:
             # !hasbit(x, n) ==> ((x >> n) & 1) ^ 1
-            expr = expression.BinOp(nmlop.DIV, expr.expr1, pow2(expr.expr2))
+            expr = expression.BinOp(nmlop.SHIFTU_RIGHT, expr.expr1, expr.expr2)
             expr = expression.BinOp(nmlop.AND, expr, expression.ConstantNumeric(1))
             expr = expression.BinOp(nmlop.XOR, expr, expression.ConstantNumeric(1))
 
@@ -416,9 +403,9 @@ class Varaction2Parser(object):
         assert isinstance(expr, expression.StorageOp)
         max = 0xF if expr.info['perm'] else 0x10F
         if isinstance(expr.register, expression.ConstantNumeric) and expr.register.value > max:
-            raise generic.ScriptError("Register number must be in range 0..%d, encountered %d." % (max, expr.register.value), expr.pos)
+            raise generic.ScriptError("Register number must be in range 0..{:d}, encountered {:d}.".format(max, expr.register.value), expr.pos)
         if expr.info['perm'] and self.feature not in (0x08, 0x0A, 0x0D):
-            raise generic.ScriptError("Persistent storage is not supported for feature '%s'" % general.feature_name(self.feature), expr.pos)
+            raise generic.ScriptError("Persistent storage is not supported for feature '{}'".format(general.feature_name(self.feature)), expr.pos)
 
         if expr.info['store']:
             op = nmlop.STO_PERM if expr.info['perm'] else nmlop.STO_TMP
@@ -445,6 +432,12 @@ class Varaction2Parser(object):
         return 0
 
     def parse_variable(self, expr):
+        """
+        Parse a variable in an expression.
+
+        @param expr:
+        @type  expr: L{expression.Variable}
+        """
         if not isinstance(expr.num, expression.ConstantNumeric):
             raise generic.ScriptError("Variable number must be a constant number", expr.num.pos)
         if not (expr.param is None or isinstance(expr.param, expression.ConstantNumeric)):
@@ -497,6 +490,8 @@ class Varaction2Parser(object):
         self.var_list.append(var)
         self.var_list_size += var.get_size()
 
+    def parse_not(self, expr):
+        self.parse_binop(expression.BinOp(nmlop.XOR, expr.expr, expression.ConstantNumeric(1)))
 
     def parse_binop(self, expr):
         if expr.op.act2_num is None: expr.supported_by_action2(True)
@@ -571,7 +566,9 @@ class Varaction2Parser(object):
                 self.parse(expr2)
                 self.var_list.append(nmlop.VAL2)
                 self.var_list_size += 1
+            # Drop the trailing VAL2 again
             self.var_list.pop()
+            self.var_list_size -= 1
         else:
             self.parse(expr)
 
@@ -587,9 +584,6 @@ class Varaction2Parser(object):
         elif isinstance(expr, expression.Boolean):
             expr = expression.BinOp(nmlop.MINU, expr.expr, expression.ConstantNumeric(1))
 
-        elif isinstance(expr, expression.Not):
-            expr = expression.BinOp(nmlop.XOR, expr.expr, expression.ConstantNumeric(1))
-
         elif isinstance(expr, expression.BinNot):
             expr = expression.BinOp(nmlop.XOR, expr.expr, expression.ConstantNumeric(0xFFFFFFFF))
 
@@ -615,6 +609,9 @@ class Varaction2Parser(object):
         elif isinstance(expr, expression.BinOp):
             self.parse_binop(expr)
 
+        elif isinstance(expr, expression.Not):
+            self.parse_not(expr)
+
         elif isinstance(expr, expression.String):
             self.parse_string(expr)
 
@@ -627,16 +624,18 @@ class Varaction2Parser(object):
             assert False #supported_by_action2 should have raised the correct error already
 
 def parse_var(info, pos):
-    res = expression.Variable(expression.ConstantNumeric(info['var']), expression.ConstantNumeric(info['start']), expression.ConstantNumeric((1 << info['size']) - 1), None, pos)
-    if 'function' in info:
-        return info['function'](res, info)
+    param = expression.ConstantNumeric(info['param']) if 'param' in info else None
+    res = expression.Variable(expression.ConstantNumeric(info['var']), expression.ConstantNumeric(info['start']),
+                              expression.ConstantNumeric((1 << info['size']) - 1), param, pos)
+    if 'value_function' in info:
+        return info['value_function'](res, info)
     return res
 
 
 def parse_60x_var(name, args, pos, info):
-    if 'function' in info:
+    if 'param_function' in info:
         # Special function to extract parameters if there is more than one
-        param, extra_params = info['function'](name, args, pos, info)
+        param, extra_params = info['param_function'](name, args, pos, info)
     else:
         # Default function to extract parameters
         param, extra_params = action2var_variables.default_60xvar(name, args, pos, info)
@@ -683,22 +682,11 @@ def parse_minmax(value, unit_str, action_list, act6, offset):
                 - Whether the resulting range may need a sanity check
     @rtype: C{tuple} of (L{ConstantNumeric} or L{SpriteGroupRef}), C{bool}
     """
-    check_range = True
     if unit_str is not None:
         raise generic.ScriptError("Using a unit is in switch-ranges is not (temporarily) not supported", value.pos)
-    elif isinstance(value, expression.ConstantNumeric):
-        result = value
-    elif isinstance(value, expression.Parameter) and isinstance(value.num, expression.ConstantNumeric):
-        act6.modify_bytes(value.num.value, 4, offset)
-        result = expression.ConstantNumeric(0)
-        check_range = False
-    else:
-        tmp_param, tmp_param_actions = actionD.get_tmp_parameter(value)
-        action_list.extend(tmp_param_actions)
-        act6.modify_bytes(tmp_param, 4, offset)
-        result = expression.ConstantNumeric(0)
-        check_range = False
-    return (result, check_range)
+    result, offset = actionD.write_action_value(value, action_list, act6, offset, 4)
+    check_range = isinstance(value, expression.ConstantNumeric)
+    return (result, offset, check_range)
 
 return_action_id = 0
 
@@ -729,7 +717,7 @@ def create_return_action(expr, feature, name, var_range):
         extra_act6.modify_bytes(mod.param, mod.size, mod.offset + 4)
     if len(extra_act6.modifications) > 0: action_list.append(extra_act6)
 
-    varaction2 = Action2Var(feature, name, var_range)
+    varaction2 = Action2Var(feature, name, expr.pos, var_range)
     varaction2.var_list = varact2parser.var_list
     varaction2.default_result = expression.ConstantNumeric(0) # Bogus result, it's the nvar == 0 that matters
     varaction2.default_comment = 'Return computed value'
@@ -738,6 +726,66 @@ def create_return_action(expr, feature, name, var_range):
     action_list.append(varaction2)
     return (action_list, ref)
 
+failed_cb_results = {}
+
+def get_failed_cb_result(feature, action_list, parent_action, pos):
+    """
+    Get a sprite group reference to use for a failed callback
+    The actions needed are created on first use, then cached in L{failed_cb_results}
+
+    @param feature: Feature to use
+    @type feature: C{int}
+
+    @param action_list: Action list to append any extra actions to
+    @type action_list: C{list} of L{BaseAction}
+
+    @param parent_action: Reference to the action of which this is a result
+    @type parent_action: L{BaseAction}
+
+    @param pos: Positional context.
+    @type  pos: L{Position}
+
+    @return: Sprite group reference to use
+    @rtype: L{SpriteGroupRef}
+    """
+    if feature in failed_cb_results:
+        varaction2 = failed_cb_results[feature]
+    else:
+        # Create action2 (+ action1, if needed)
+        # Import here to avoid circular imports
+        from nml.actions import action1, action2layout, action2production, action2real
+        if feature == 0x0A:
+            # Industries -> production action2
+            act2 = action2production.make_empty_production_action2(pos)
+        elif feature in (0x07, 0x09, 0x0F, 0x11):
+            # Tile layout action2
+            act2 = action2layout.make_empty_layout_action2(feature, pos)
+        else:
+            # Normal action2
+            act1_actions, act1_index = action1.make_cb_failure_action1(feature)
+            action_list.extend(act1_actions)
+            act2 = action2real.make_simple_real_action2(feature, "@CB_FAILED_REAL{:02X}".format(feature), pos, act1_index)
+        action_list.append(act2)
+
+        # Create varaction2, to choose between returning graphics and 0, depending on CB
+        varact2parser = Varaction2Parser(feature)
+        varact2parser.parse_expr(expression.Variable(expression.ConstantNumeric(0x0C), mask=expression.ConstantNumeric(0xFFFF)))
+
+        varaction2 = Action2Var(feature, "@CB_FAILED{:02X}".format(feature), pos, 0x89)
+        varaction2.var_list = varact2parser.var_list
+
+        varaction2.ranges.append(VarAction2Range(expression.ConstantNumeric(0), expression.ConstantNumeric(0), expression.ConstantNumeric(0), "graphics callback -> return 0"))
+        varaction2.default_result = expression.SpriteGroupRef(expression.Identifier(act2.name), [], None, act2)
+        varaction2.default_comment = "Non-graphics callback, return graphics result"
+        action2.add_ref(varaction2.default_result, varaction2)
+
+        action_list.append(varaction2)
+        failed_cb_results[feature] = varaction2
+
+    ref = expression.SpriteGroupRef(expression.Identifier(varaction2.name), [], None, varaction2)
+    action2.add_ref(ref, parent_action)
+    return ref
+
 def parse_sg_ref_result(result, action_list, parent_action, var_range):
     """
     Parse a result that is a sprite group reference.
@@ -757,6 +805,9 @@ def parse_sg_ref_result(result, action_list, parent_action, var_range):
     @return: Result to use in the calling varaction2
     @rtype: L{SpriteGroupRef}
     """
+    if result.name.value == "CB_FAILED":
+        return get_failed_cb_result(parent_action.feature, action_list, parent_action, result.pos)
+
     if len(result.param_list) == 0:
         action2.add_ref(result, parent_action)
         return result
@@ -783,8 +834,8 @@ def parse_sg_ref_result(result, action_list, parent_action, var_range):
     if len(extra_act6.modifications) > 0: action_list.append(extra_act6)
 
     global return_action_id
-    name = "@return_action_%d" % return_action_id
-    varaction2 = Action2Var(parent_action.feature, name, var_range)
+    name = "@return_action_{:d}".format(return_action_id)
+    varaction2 = Action2Var(parent_action.feature, name, result.pos, var_range)
     return_action_id += 1
     varaction2.var_list = varact2parser.var_list
     ref = expression.SpriteGroupRef(result.name, [], result.pos)
@@ -793,11 +844,14 @@ def parse_sg_ref_result(result, action_list, parent_action, var_range):
     varaction2.default_comment = result.name.value
     # Add the references as procs, to make sure, that any intermediate registers
     # are freed at the spritelayout and thus not selected to pass parameters
+    # Reference is used twice (range + default) so call add_ref twice
     action2.add_ref(ref, varaction2, True)
     action2.add_ref(ref, varaction2, True)
 
     ref = expression.SpriteGroupRef(expression.Identifier(name), [], None, varaction2)
     action_list.append(varaction2)
+    action2.add_ref(ref, parent_action)
+
     return ref
 
 def parse_result(value, action_list, act6, offset, parent_action, none_result, var_range, repeat_result = 1):
@@ -841,38 +895,44 @@ def parse_result(value, action_list, act6, offset, parent_action, none_result, v
         else:
             result = none_result
     elif isinstance(value, expression.SpriteGroupRef):
-        comment = value.name.value + ';'
         result = parse_sg_ref_result(value, action_list, parent_action, var_range)
+        comment = result.name.value + ';'
     elif isinstance(value, expression.ConstantNumeric):
-        comment = "return %d;" % value.value
+        comment = "return {:d};".format(value.value)
         result = value
+        if not(-16384 <= value.value <= 32767):
+            msg = "Callback results are limited to -16384..16383 (when the result is a signed number) or 0..32767 (unsigned), encountered {:d}."
+            msg = msg.format(value.value)
+            raise generic.ScriptError(msg, value.pos)
+
     elif isinstance(value, expression.String):
-        comment = "return %s;" % str(value)
+        comment = "return {};".format(str(value))
         str_id, actions = action4.get_string_action4s(0, 0xD0, value)
         action_list.extend(actions)
         result = expression.ConstantNumeric(str_id - 0xD000 + 0x8000)
     elif value.supported_by_actionD(False):
         tmp_param, tmp_param_actions = actionD.get_tmp_parameter(expression.BinOp(nmlop.OR, value, expression.ConstantNumeric(0x8000)).reduce())
-        comment = "return param[%d];" % tmp_param
+        comment = "return param[{:d}];".format(tmp_param)
         action_list.extend(tmp_param_actions)
         for i in range(repeat_result):
             act6.modify_bytes(tmp_param, 2, offset + 2*i)
         result = expression.ConstantNumeric(0)
     else:
         global return_action_id
-        extra_actions, result = create_return_action(value, parent_action.feature, "@return_action_%d" % return_action_id, var_range)
+        extra_actions, result = create_return_action(value, parent_action.feature, "@return_action_{:d}".format(return_action_id), var_range)
         return_action_id += 1
         action2.add_ref(result, parent_action)
         action_list.extend(extra_actions)
-        comment = "return %s" % str(value)
+        comment = "return {}".format(value)
     return (result, comment)
 
 def get_feature(switch_block):
-    feature = switch_block.feature_set.copy().pop()
+    feature = next(iter(switch_block.feature_set))
     if switch_block.var_range == 0x8A:
         feature = action2var_variables.varact2parent_scope[feature]
         if feature is None:
             raise generic.ScriptError("Parent scope for this feature not available, feature: " + str(feature), switch_block.pos)
+
     return feature
 
 def reduce_varaction2_expr(expr, feature, extra_dicts = []):
@@ -885,8 +945,6 @@ def reduce_varaction2_expr(expr, feature, extra_dicts = []):
     # TODO pass the function name along to avoid this
     func60x = lambda value, pos: expression.FunctionPtr(expression.Identifier(generic.reverse_lookup(vars_60x, value), pos), parse_60x_var, value)
 
-    # make sure, that variables take precedence about global constants / parameters
-    # this way, use the current climate instead of the climate at load time.
     return expr.reduce(extra_dicts + [(action2var_variables.varact2_globalvars, parse_var), \
         (vars_normal, parse_var), \
         (vars_60x, func60x)] + \
@@ -900,8 +958,8 @@ def parse_varaction2(switch_block):
     act6 = action6.Action6()
     action_list = action2real.create_spriteset_actions(switch_block)
 
-    feature = switch_block.feature_set.copy().pop()
-    varaction2 = Action2Var(feature, switch_block.name.value, switch_block.var_range)
+    feature = next(iter(switch_block.feature_set))
+    varaction2 = Action2Var(feature, switch_block.name.value, switch_block.pos, switch_block.var_range)
 
     expr = reduce_varaction2_expr(switch_block.expr, get_feature(switch_block))
 
@@ -916,7 +974,7 @@ def parse_varaction2(switch_block):
     offset += parser.var_list_size + 1 # +1 for the byte num-ranges
 
     none_result = None
-    if any(map(lambda x: x is None, [r.result for r in switch_block.body.ranges] + [switch_block.body.default])):
+    if any(x is not None and x.value is None for x in [r.result for r in switch_block.body.ranges] + [switch_block.body.default]):
         # Computed result is returned in at least one result
         if len(switch_block.body.ranges) == 0:
             # There is only a default, which is 'return computed result', so we're fine
@@ -927,26 +985,16 @@ def parse_varaction2(switch_block):
                     feature, switch_block.name.value + "@return", 0x89)
             action_list.extend(extra_actions)
 
-    if len(switch_block.body.ranges) == 0 and switch_block.body.default is not None:
-        # Computed result is not returned, but there are no ranges
-        # Add one range, to avoid the nvar == 0 bear trap
-        from nml.ast import switch
-        ranges_list = [switch.SwitchRange(expression.ConstantNumeric(0), expression.ConstantNumeric(0), switch_block.body.default)]
-    else:
-        ranges_list = switch_block.body.ranges
-
     used_ranges = []
-    for r in ranges_list:
+    for r in switch_block.body.ranges:
         comment = str(r.min) + " .. " + str(r.max) + ": "
 
-        range_result, range_comment = parse_result(r.result, action_list, act6, offset, varaction2, none_result, switch_block.var_range)
+        range_result, range_comment = parse_result(r.result.value, action_list, act6, offset, varaction2, none_result, switch_block.var_range)
         comment += range_comment
         offset += 2 # size of result
 
-        range_min, check_min = parse_minmax(r.min, r.unit, action_list, act6, offset)
-        offset += 4
-        range_max, check_max = parse_minmax(r.max, r.unit, action_list, act6, offset)
-        offset += 4
+        range_min, offset, check_min = parse_minmax(r.min, r.unit, action_list, act6, offset)
+        range_max, offset, check_max = parse_minmax(r.max, r.unit, action_list, act6, offset)
 
         range_overlap = False
         if check_min and check_max:
@@ -969,9 +1017,33 @@ def parse_varaction2(switch_block):
         if not range_overlap:
             varaction2.ranges.append(VarAction2Range(range_min, range_max, range_result, comment))
 
-    default, default_comment = parse_result(switch_block.body.default, action_list, act6, offset, varaction2, none_result, switch_block.var_range)
+    if len(switch_block.body.ranges) == 0 and \
+            (switch_block.body.default is None or switch_block.body.default.value is not None):
+        # Computed result is not returned, but there are no ranges
+        # Add one range, to avoid the nvar == 0 bear trap
+        offset += 10
+        varaction2.ranges.append(VarAction2Range(expression.ConstantNumeric(1), expression.ConstantNumeric(0),
+                expression.ConstantNumeric(0), "Bogus range to avoid nvar == 0"))
+
+    # Handle default result
+    if switch_block.body.default is not None:
+        # there is a default value
+        default_result = switch_block.body.default.value
+    else:
+        # Default to CB_FAILED
+        default_result = expression.SpriteGroupRef(expression.Identifier('CB_FAILED', None), [], None)
+
+    default, default_comment = parse_result(default_result, action_list, act6, offset, varaction2, none_result, switch_block.var_range)
     varaction2.default_result = default
-    varaction2.default_comment = 'Return computed value' if switch_block.body.default is None else 'default: ' + default_comment
+    if switch_block.body.default is None:
+        varaction2.default_comment = 'No default specified -> fail callback'
+    elif switch_block.body.default.value is None:
+        varaction2.default_comment = 'Return computed value'
+    else:
+        varaction2.default_comment = 'default: ' + default_comment
+
+
+
 
     if len(act6.modifications) > 0: action_list.append(act6)
 
