@@ -22,7 +22,7 @@ class Action5(base_action.BaseAction):
         self.num_sprites = num_sprites
         self.offset = offset
 
-    def prepare_output(self):
+    def prepare_output(self, sprite_num):
         if self.offset is not None:
             self.type |= 0x80
 
@@ -75,13 +75,14 @@ action5_table = {
     'AQUEDUCTS' : (0x12, 8, Action5BlockType.OFFSET),
     'AUTORAIL' : (0x13, 55, Action5BlockType.OFFSET),
     'FLAGS' : (0x14, 36, Action5BlockType.OFFSET),
-    'OTTD_GUI' : (0x15, 166, Action5BlockType.OFFSET),
+    'OTTD_GUI' : (0x15, 175, Action5BlockType.OFFSET),
     'AIRPORT_PREVIEW' : (0x16, 9, Action5BlockType.OFFSET),
     'RAILTYPE_TUNNELS': (0x17, 16, Action5BlockType.OFFSET),
+    'OTTD_RECOLOUR' : (0x18, 1, Action5BlockType.OFFSET),
 }
 
 def parse_action5(replaces):
-    real_sprite_list = real_sprite.parse_sprite_list(replaces.sprite_list, replaces.pcx, block_name = replaces.name)
+    real_sprite_list = real_sprite.parse_sprite_data(replaces)
     num_sprites = len(real_sprite_list)
 
     if replaces.type.value not in action5_table:
@@ -91,20 +92,34 @@ def parse_action5(replaces):
 
     if block_type == Action5BlockType.FIXED:
         if num_sprites < num_required:
-            raise generic.ScriptError("Invalid sprite count for sprite replacement type '%s', expcected %d, got %d" % (replaces.type, num_required, num_sprites), replaces.pos)
+            msg = "Invalid sprite count for sprite replacement type '{}', expected {:d}, got {:d}"
+            msg = msg.format(replaces.type, num_required, num_sprites)
+            raise generic.ScriptError(msg, replaces.pos)
+
         elif num_sprites > num_required:
-            generic.print_warning("Too many sprites specified for sprite replacement type '%s', expcected %d, got %d, extra sprites may be ignored" % (replaces.type, num_required, num_sprites), replaces.pos)
+            msg = "Too many sprites specified for sprite replacement type '{}', expected {:d}, got {:d}, extra sprites may be ignored"
+            msg = msg.format(replaces.type, num_required, num_sprites)
+            generic.print_warning(msg, replaces.pos)
+
         if replaces.offset != 0:
-            raise generic.ScriptError("replacenew parameter 'offset' must be zero for sprite replacement type '%s'" % replaces.type, replaces.pos)
+            msg = "replacenew parameter 'offset' must be zero for sprite replacement type '{}'".format(replaces.type)
+            raise generic.ScriptError(msg, replaces.pos)
+
     elif block_type == Action5BlockType.ANY:
         if replaces.offset != 0:
-            raise generic.ScriptError("replacenew parameter 'offset' must be zero for sprite replacement type '%s'" % replaces.type, replaces.pos)
+            msg = "replacenew parameter 'offset' must be zero for sprite replacement type '{}'".format(replaces.type)
+            raise generic.ScriptError(msg, replaces.pos)
+
     elif block_type == Action5BlockType.OFFSET:
         if num_sprites + replaces.offset > num_required:
-            generic.print_warning("Exceeding the limit of %d sprites for sprite replacement type '%s', extra sprites may be ignored" % (num_required, replaces.type), replaces.pos)
+            msg = "Exceeding the limit of {:d} sprites for sprite replacement type '{}', extra sprites may be ignored"
+            msg = msg.format(num_required, replaces.type)
+            generic.print_warning(msg, replaces.pos)
+
         if replaces.offset != 0 or num_sprites != num_required:
             offset = replaces.offset
     else:
         assert 0
 
     return [Action5(type_id, num_sprites, offset)] + real_sprite_list
+
