@@ -14,8 +14,8 @@ with NML; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA."""
 
 from nml import expression, generic, global_constants
-from nml.actions import action1, action2, action2layout, action2real, real_sprite
-from nml.ast import base_statement
+from nml.actions import action2, action2layout, action2real, real_sprite
+from nml.ast import base_statement, sprite_container
 
 class TemplateDeclaration(base_statement.BaseStatement):
     def __init__(self, name, param_list, sprite_list, pos):
@@ -23,40 +23,40 @@ class TemplateDeclaration(base_statement.BaseStatement):
         self.name = name
         self.param_list = param_list
         self.sprite_list = sprite_list
-    
+
     def pre_process(self):
         #check that all templates that are referred to exist at this point
         #This prevents circular dependencies
         for sprite in self.sprite_list:
             if isinstance(sprite, real_sprite.TemplateUsage):
                 if sprite.name.value == self.name.value:
-                    raise generic.ScriptError("Sprite template '%s' includes itself." % sprite.name.value, self.pos)
+                    raise generic.ScriptError("Sprite template '{}' includes itself.".format(sprite.name.value), self.pos)
                 elif sprite.name.value not in real_sprite.sprite_template_map:
                     raise generic.ScriptError("Encountered unknown template identifier: " + sprite.name.value, sprite.pos)
         #Register template
         if self.name.value not in real_sprite.sprite_template_map:
             real_sprite.sprite_template_map[self.name.value] = self
         else:
-            raise generic.ScriptError("Template named '%s' is already defined, first definition at %s" % (self.name.value, real_sprite.sprite_template_map[self.name.value].pos), self.pos)
+            raise generic.ScriptError("Template named '{}' is already defined, first definition at {}".format(self.name.value, real_sprite.sprite_template_map[self.name.value].pos), self.pos)
 
     def get_labels(self):
         labels = {}
         offset = 0
         for sprite in self.sprite_list:
             sprite_labels, num_sprites = sprite.get_labels()
-            for lbl, lbl_offset in sprite_labels.iteritems():
+            for lbl, lbl_offset in list(sprite_labels.items()):
                 if lbl in labels:
-                    raise generic.ScriptError("Duplicate label encountered; '%s' already exists." % lbl, self.pos)
+                    raise generic.ScriptError("Duplicate label encountered; '{}' already exists.".format(lbl), self.pos)
                 labels[lbl] = lbl_offset + offset
             offset += num_sprites
         return labels, offset
 
     def debug_print(self, indentation):
-        print indentation*' ' + 'Template declaration:', self.name.value
-        print (indentation+2)*' ' + 'Parameters:'
+        generic.print_dbg(indentation, 'Template declaration:', self.name.value)
+        generic.print_dbg(indentation + 2, 'Parameters:')
         for param in self.param_list:
             param.debug_print(indentation + 4)
-        print (indentation+2)*' ' + 'Sprites:'
+        generic.print_dbg(indentation + 2, 'Sprites:')
         for sprite in self.sprite_list:
             sprite.debug_print(indentation + 4)
 
@@ -64,15 +64,15 @@ class TemplateDeclaration(base_statement.BaseStatement):
         return []
 
     def __str__(self):
-        ret = "template %s(%s) {\n" % (str(self.name), ", ".join([str(param) for param in self.param_list]))
+        ret = "template {}({}) {{\n".format(str(self.name), ", ".join([str(param) for param in self.param_list]))
         for sprite in self.sprite_list:
-            ret += "\t%s\n" % str(sprite)
+            ret += "\t{}\n".format(sprite)
         ret += "}\n"
         return ret
 
 spriteset_base_class = action2.make_sprite_group_class(True, True, False, cls_is_relocatable = True)
 
-class SpriteSet(spriteset_base_class):
+class SpriteSet(spriteset_base_class, sprite_container.SpriteContainer):
     def __init__(self, param_list, sprite_list, pos):
         base_statement.BaseStatement.__init__(self, "spriteset", pos, False, False)
         if not (1 <= len(param_list) <= 2):
@@ -80,25 +80,28 @@ class SpriteSet(spriteset_base_class):
         name = param_list[0]
         if not isinstance(name, expression.Identifier):
             raise generic.ScriptError("Spriteset parameter 1 'name' should be an identifier", name.pos)
+        sprite_container.SpriteContainer.__init__(self, "spriteset", name)
         self.initialize(name)
+
         if len(param_list) >= 2:
-            self.pcx = param_list[1].reduce()
-            if not isinstance(self.pcx, expression.StringLiteral):
-                raise generic.ScriptError("Spriteset-block parameter 2 'file' must be a string literal", self.pcx.pos)
+            self.image_file = param_list[1].reduce()
+            if not isinstance(self.image_file, expression.StringLiteral):
+                raise generic.ScriptError("Spriteset-block parameter 2 'file' must be a string literal", self.image_file.pos)
         else:
-            self.pcx = None
+            self.image_file = None
         self.sprite_list = sprite_list
         self.action1_num = None #set number in action1
         self.labels = {} #mapping of real sprite labels to offsets
+        self.add_sprite_data(self.sprite_list, self.image_file, pos)
 
     def pre_process(self):
         spriteset_base_class.pre_process(self)
         offset = 0
         for sprite in self.sprite_list:
             sprite_labels, num_sprites = sprite.get_labels()
-            for lbl, lbl_offset in sprite_labels.iteritems():
+            for lbl, lbl_offset in list(sprite_labels.items()):
                 if lbl in self.labels:
-                    raise generic.ScriptError("Duplicate label encountered; '%s' already exists." % lbl, self.pos)
+                    raise generic.ScriptError("Duplicate label encountered; '{}' already exists.".format(lbl), self.pos)
                 self.labels[lbl] = lbl_offset + offset
             offset += num_sprites
 
@@ -106,9 +109,10 @@ class SpriteSet(spriteset_base_class):
         return []
 
     def debug_print(self, indentation):
-        print indentation*' ' + 'Sprite set:', self.name.value
-        print (indentation+2)*' ' + 'Source:  ', self.pcx.value if self.pcx is not None else 'None'
-        print (indentation+2)*' ' + 'Sprites:'
+        generic.print_dbg(indentation, 'Sprite set:', self.name.value)
+        generic.print_dbg(indentation + 2, 'Source:  ', self.image_file.value if self.image_file is not None else 'None')
+
+        generic.print_dbg(indentation + 2, 'Sprites:')
         for sprite in self.sprite_list:
             sprite.debug_print(indentation + 4)
 
@@ -117,10 +121,10 @@ class SpriteSet(spriteset_base_class):
         return []
 
     def __str__(self):
-        filename = (", " + str(self.pcx)) if self.pcx is not None else ""
-        ret = "spriteset(%s%s) {\n" % (self.name, filename)
+        filename = (", " + str(self.image_file)) if self.image_file is not None else ""
+        ret = "spriteset({}{}) {{\n".format(self.name, filename)
         for sprite in self.sprite_list:
-            ret += "\t%s\n" % str(sprite)
+            ret += "\t{}\n".format(str(sprite))
         ret += "}\n"
         return ret
 
@@ -141,21 +145,21 @@ class SpriteGroup(spritegroup_base_class):
         return []
 
     def debug_print(self, indentation):
-        print indentation*' ' + 'Sprite group:', self.name.value
+        generic.print_dbg(indentation, 'Sprite group:', self.name.value)
         for spriteview in self.spriteview_list:
             spriteview.debug_print(indentation + 2)
 
     def get_action_list(self):
         action_list = []
-        if self.prepare_output():
+        if self.prepare_act2_output():
             for feature in sorted(self.feature_set):
                 action_list.extend(action2real.get_real_action2s(self, feature))
         return action_list
 
     def __str__(self):
-        ret = "spritegroup %s {\n" % (self.name)
+        ret = "spritegroup {} {{\n".format(self.name)
         for spriteview in self.spriteview_list:
-            ret += "\t%s\n" % str(spriteview)
+            ret += "\t{}\n".format(spriteview)
         ret += "}\n"
         return ret
 
@@ -175,13 +179,13 @@ class SpriteView(object):
                 raise generic.ScriptError("Spritesets referenced from a spritegroup may not have parameters.", sg_ref.pos)
 
     def debug_print(self, indentation):
-        print indentation*' ' + 'Sprite view:', self.name.value
-        print (indentation+2)*' ' + 'Sprite sets:'
+        generic.print_dbg(indentation, 'Sprite view:', self.name.value)
+        generic.print_dbg(indentation + 2, 'Sprite sets:')
         for spriteset in self.spriteset_list:
             spriteset.debug_print(indentation + 4)
 
     def __str__(self):
-        return "%s: [%s];" % (str(self.name), ", ".join([str(spriteset) for spriteset in self.spriteset_list]))
+        return "{}: [{}];".format(self.name, ", ".join([str(spriteset) for spriteset in self.spriteset_list]))
 
 spritelayout_base_class = action2.make_sprite_group_class(False, True, False)
 
@@ -202,7 +206,7 @@ class SpriteLayout(spritelayout_base_class):
             if not isinstance(param, expression.Identifier):
                 raise generic.ScriptError("spritelayout parameter names must be identifiers.", param.pos)
             if param.value in seen_names:
-                raise generic.ScriptError("Duplicate parameter name '%s' encountered." % param.value, param.pos)
+                raise generic.ScriptError("Duplicate parameter name '{}' encountered.".format(param.value), param.pos)
             seen_names.add(param.value)
         spritelayout_base_class.pre_process(self)
 
@@ -210,22 +214,23 @@ class SpriteLayout(spritelayout_base_class):
         return []
 
     def debug_print(self, indentation):
-        print indentation*' ' + 'Sprite layout:', self.name.value
-        print (indentation+2)*' ' + 'Parameters:'
+        generic.print_dbg(indentation, 'Sprite layout:', self.name.value)
+        generic.print_dbg(indentation + 2, 'Parameters:')
         for param in self.param_list:
             param.debug_print(indentation + 4)
-        print (indentation+2)*' ' + 'Sprites:'
+        generic.print_dbg(indentation + 2, 'Sprites:')
         for layout_sprite in self.layout_sprite_list:
             layout_sprite.debug_print(indentation + 4)
 
     def __str__(self):
-        return 'spritelayout %s {\n%s\n}\n' % (str(self.name), '\n'.join([str(x) for x in self.layout_sprite_list]))
+        params = "" if not self.param_list else "({})".format(", ".join(str(x) for x in self.param_list))
+        return 'spritelayout {}{} {{\n{}\n}}\n'.format(str(self.name), params, '\n'.join(str(x) for x in self.layout_sprite_list))
 
     def get_action_list(self):
         action_list = []
-        if self.prepare_output():
+        if self.prepare_act2_output():
             for feature in sorted(self.feature_set):
-                action_list.extend(action2layout.get_layout_action2s(self, feature))
+                action_list.extend(action2layout.get_layout_action2s(self, feature, self.pos))
         return action_list
 
 class LayoutSprite(object):
@@ -235,9 +240,9 @@ class LayoutSprite(object):
         self.pos = pos
 
     def debug_print(self, indentation):
-        print indentation*' ' + 'Tile layout sprite of type:', self.type
+        generic.print_dbg(indentation, 'Tile layout sprite of type:', self.type)
         for layout_param in self.param_list:
             layout_param.debug_print(indentation + 2)
 
     def __str__(self):
-        return '\t%s {\n\t\t%s\n\t}' % (self.type, '\n\t\t'.join([str(layout_param) for layout_param in self.param_list]))
+        return '\t{} {{\n\t\t{}\n\t}}'.format(self.type, '\n\t\t'.join(str(layout_param) for layout_param in self.param_list))
