@@ -35,7 +35,7 @@ class Switch(switch_base_class):
         if param_list[1].value in var_ranges:
             self.var_range = var_ranges[param_list[1].value]
         else:
-            raise generic.ScriptError("Unrecognized value for switch parameter 2 'variable range': '%s'" % param_list[1].value, param_list[1].pos)
+            raise generic.ScriptError("Unrecognized value for switch parameter 2 'variable range': '{}'".format(param_list[1].value), param_list[1].pos)
         if not isinstance(param_list[2], expression.Identifier):
             raise generic.ScriptError("Switch-block parameter 3 'name' must be an identifier.", param_list[2].pos)
         self.initialize(param_list[2], general.parse_feature(param_list[0]).value)
@@ -51,25 +51,27 @@ class Switch(switch_base_class):
     def collect_references(self):
         all_refs = []
         for result in [r.result for r in self.body.ranges] + [self.body.default]:
-            if isinstance(result, expression.SpriteGroupRef):
-                all_refs.append(result)
+            if result is not None and isinstance(result.value, expression.SpriteGroupRef):
+                all_refs.append(result.value)
         return all_refs
 
     def debug_print(self, indentation):
-        print indentation*' ' + 'Switch, Feature = %d, name = %s', (self.feature_set.copy().pop(), self.name.value)
-        print (2+indentation)*' ' + 'Expression:'
+        generic.print_dbg(indentation, 'Switch, Feature = {:d}, name = {}'.format(next(iter(self.feature_set)), self.name.value))
+
+        generic.print_dbg(indentation + 2, 'Expression:')
         self.expr.debug_print(indentation + 4)
-        print (2+indentation)*' ' + 'Body:'
+
+        generic.print_dbg(indentation + 2, 'Body:')
         self.body.debug_print(indentation + 4)
 
     def get_action_list(self):
-        if self.prepare_output():
+        if self.prepare_act2_output():
             return action2var.parse_varaction2(self)
         return []
 
     def __str__(self):
         var_range = 'SELF' if self.var_range == 0x89 else 'PARENT'
-        return 'switch(%s, %s, %s, %s) {\n%s}\n' % (str(self.feature_set.copy().pop()), var_range, str(self.name), str(self.expr), str(self.body))
+        return 'switch({}, {}, {}, {}) {{\n{}}}\n'.format(str(next(iter(self.feature_set))), var_range, str(self.name), str(self.expr), str(self.body))
 
 
 class SwitchBody(object):
@@ -81,37 +83,36 @@ class SwitchBody(object):
     @type ranges: C{list} of L{SwitchRange}
 
     @ivar default: Default result to use if no range matches
-    @type default: L{SpriteGroupRef}, L{Expression} or C{None} (before pre-processing only), depending on the type of result.
+    @type default: L{SwitchValue} or C{None} if N/A
     """
     def __init__(self, ranges, default):
         self.ranges = ranges
         self.default = default
 
     def reduce_expressions(self, var_feature):
-        if self.default is not None:
-            self.default = action2var.reduce_varaction2_expr(self.default, var_feature)
-        for r in self.ranges:
-            r.reduce_expressions(var_feature)
+        for r in self.ranges[:]:
+            if r.min is r.max and isinstance(r.min, expression.Identifier) and r.min.value == 'default':
+                if self.default is not None:
+                    raise generic.ScriptError("Switch-block has more than one default, which is impossible.", r.result.pos)
+                self.default = r.result
+                self.ranges.remove(r)
+            else:
+                r.reduce_expressions(var_feature)
+        if self.default is not None and self.default.value is not None:
+            self.default.value = action2var.reduce_varaction2_expr(self.default.value, var_feature)
+
 
     def debug_print(self, indentation):
         for r in self.ranges:
             r.debug_print(indentation)
-        print indentation*' ' + 'Default:'
         if self.default is not None:
+            generic.print_dbg(indentation, 'Default:')
             self.default.debug_print(indentation + 2)
-        else:
-            print (indentation+2)*' ' + 'Return computed value'
 
     def __str__(self):
-        ret = ''
-        for r in self.ranges:
-            ret += '\t%s\n' % str(r)
-        if self.default is None:
-            ret += '\treturn;\n'
-        elif isinstance(self.default, expression.SpriteGroupRef):
-            ret += '\t%s;\n' % str(self.default)
-        else:
-            ret += '\treturn %s;\n' % str(self.default)
+        ret = ''.join('\t{}\n'.format(r) for r in self.ranges)
+        if self.default is not None:
+            ret += '\t{}\n'.format(str(self.default))
         return ret
 
 class SwitchRange(object):
@@ -124,37 +125,65 @@ class SwitchRange(object):
     def reduce_expressions(self, var_feature):
         self.min = self.min.reduce(global_constants.const_list)
         self.max = self.max.reduce(global_constants.const_list)
-        if self.result is not None:
-            self.result = action2var.reduce_varaction2_expr(self.result, var_feature)
+        if self.result.value is not None:
+            self.result.value = action2var.reduce_varaction2_expr(self.result.value, var_feature)
 
     def debug_print(self, indentation):
-        print indentation*' ' + 'Min:'
+        generic.print_dbg(indentation, 'Min:')
         self.min.debug_print(indentation + 2)
-        print indentation*' ' + 'Max:'
+        generic.print_dbg(indentation, 'Max:')
         self.max.debug_print(indentation + 2)
-        print indentation*' ' + 'Result:'
-        if self.result is not None:
-            self.result.debug_print(indentation + 2)
-        else:
-            print (indentation+2)*' ' + 'Return computed value'
+        generic.print_dbg(indentation, 'Result:')
+        self.result.debug_print(indentation + 2)
 
     def __str__(self):
         ret = str(self.min)
         if not isinstance(self.min, expression.ConstantNumeric) or not isinstance(self.max, expression.ConstantNumeric) or self.max.value != self.min.value:
             ret += '..' + str(self.max)
-        if self.result is None:
-            ret += ': return;'
-        elif isinstance(self.result, expression.SpriteGroupRef):
-            ret += ': %s;' % str(self.result)
-        else:
-            ret += ': return %s;' % str(self.result)
+        ret += ': ' + str(self.result)
         return ret
 
+class SwitchValue(object):
+    """
+    Class representing a single returned value or sprite group in a switch-block
+    Also used for random-switch and graphics blocks
+
+    @ivar value: Value to return
+    @type value: L{Expression} or C{None}
+
+    @ivar is_return: Whether the return keyword was present
+    @type is_return: C{bool}
+
+    @ivar pos: Position information
+    @type pos: L{Position}
+    """
+    def __init__(self, value, is_return, pos):
+        self.value = value
+        self.is_return = is_return
+        self.pos = pos
+
+    def debug_print(self, indentation):
+        if self.value is None:
+            assert self.is_return
+            generic.print_dbg(indentation, 'Return computed value')
+        else:
+            generic.print_dbg(indentation, 'Return value:' if self.is_return else 'Go to block:')
+            self.value.debug_print(indentation + 2)
+
+    def __str__(self):
+        if self.value is None:
+            assert self.is_return
+            return 'return;'
+        elif self.is_return:
+            return 'return {};'.format(self.value)
+        else:
+            return '{};'.format(self.value)
+
 class RandomSwitch(switch_base_class):
     def __init__(self, param_list, choices, pos):
         base_statement.BaseStatement.__init__(self, "random_switch-block", pos, False, False)
         if not (3 <= len(param_list) <= 4):
-            raise generic.ScriptError("random_switch requires 3 or 4 parameters, encountered %d" % len(param_list), pos)
+            raise generic.ScriptError("random_switch requires 3 or 4 parameters, encountered {:d}".format(len(param_list)), pos)
         #feature
         feature = general.parse_feature(param_list[0]).value
 
@@ -203,17 +232,19 @@ class RandomSwitch(switch_base_class):
 
     def pre_process(self):
         for choice in self.choices:
-            choice.reduce_expressions(self.feature_set.copy().pop())
+            choice.reduce_expressions(next(iter(self.feature_set)))
 
         for dep_list in (self.dependent, self.independent):
             for i, dep in enumerate(dep_list[:]):
-                dep_list[i] = dep.reduce(global_constants.const_list)
+                if dep.is_return:
+                    raise generic.ScriptError("Expected a random_switch identifier after (in)dependent, not a return.", dep.pos)
+                dep_list[i] = dep.value.reduce(global_constants.const_list)
                 # Make sure, all [in]dependencies refer to existing random switch blocks
                 if (not isinstance(dep_list[i], expression.SpriteGroupRef)) or len(dep_list[i].param_list) > 0:
                     raise generic.ScriptError("Value for (in)dependent should be an identifier", dep_list[i].pos)
                 spritegroup = action2.resolve_spritegroup(dep_list[i].name)
                 if not isinstance(spritegroup, RandomSwitch):
-                    raise generic.ScriptError("Value of (in)dependent '%s' should refer to a random_switch." % dep_list[i].name.value, dep_list[i].pos)
+                    raise generic.ScriptError("Value of (in)dependent '{}' should refer to a random_switch.".format(dep_list[i].name.value), dep_list[i].pos)
 
         self.triggers = self.triggers.reduce_constant(global_constants.const_list)
         if not (0 <= self.triggers.value <= 255):
@@ -225,39 +256,41 @@ class RandomSwitch(switch_base_class):
     def collect_references(self):
         all_refs = []
         for choice in self.choices:
-            if isinstance(choice.result, expression.SpriteGroupRef):
-                all_refs.append(choice.result)
+            if isinstance(choice.result.value, expression.SpriteGroupRef):
+                all_refs.append(choice.result.value)
         return all_refs
 
     def debug_print(self, indentation):
-        print indentation*' ' + 'Random'
-        print (2+indentation)*' ' + 'Feature:', self.feature_set.copy().pop()
-        print (2+indentation)*' ' + 'Type:'
+        generic.print_dbg(indentation, 'Random')
+        generic.print_dbg(indentation + 2, 'Feature:', next(iter(self.feature_set)))
+        generic.print_dbg(indentation + 2, 'Type:')
         self.type.debug_print(indentation + 4)
-        print (2+indentation)*' ' + 'Name:', self.name.value
-        print (2+indentation)*' ' + 'Triggers:'
+        generic.print_dbg(indentation + 2, 'Name:', self.name.value)
+
+        generic.print_dbg(indentation + 2, 'Triggers:')
         self.triggers.debug_print(indentation + 4)
         for dep in self.dependent:
-            print (2+indentation)*' ' + 'Dependent on:'
+            generic.print_dbg(indentation + 2, 'Dependent on:')
             dep.debug_print(indentation + 4)
         for indep in self.independent:
-            print (2+indentation)*' ' + 'Independent from:'
+            generic.print_dbg(indentation + 2, 'Independent from:')
             indep.debug_print(indentation + 4)
-        print (2+indentation)*' ' + 'Choices:'
+
+        generic.print_dbg(indentation + 2, 'Choices:')
         for choice in self.choices:
             choice.debug_print(indentation + 4)
 
     def get_action_list(self):
-        if self.prepare_output():
+        if self.prepare_act2_output():
             return action2random.parse_randomswitch(self)
         return []
 
     def __str__(self):
-        ret = 'random_switch(%s, %s, %s, %s) {\n' % (str(self.feature_set.copy().pop()), str(self.type), str(self.name), str(self.triggers))
+        ret = 'random_switch({}, {}, {}, {}) {{\n'.format(str(next(iter(self.feature_set))), str(self.type), str(self.name), str(self.triggers))
         for dep in self.dependent:
-            ret += 'dependent: %s;\n' % str(dep)
+            ret += 'dependent: {};\n'.format(dep)
         for indep in self.independent:
-            ret += 'independent: %s;\n' % str(indep)
+            ret += 'independent: {};\n'.format(indep)
         for choice in self.choices:
             ret += str(choice) + '\n'
         ret += '}\n'
@@ -271,31 +304,27 @@ class RandomChoice(object):
     @type probability: L{Expression}
 
     @ivar result: Result of this choice, either another action2 or a return value
-    @type result: L{SpriteGroupRef} or L{Expression}
+    @type result: L{SwitchValue}
     """
     def __init__ (self, probability, result):
         self.probability = probability
-        if result is None:
-            raise generic.ScriptError("Returning the computed value is not possible in a random_switch, as there is no computed value.", self.probability.pos)
+        if result.value is None:
+            raise generic.ScriptError("Returning the computed value is not possible in a random_switch, as there is no computed value.", result.pos)
         self.result = result
 
     def reduce_expressions(self, var_feature):
         self.probability = self.probability.reduce_constant(global_constants.const_list)
         if self.probability.value <= 0:
             raise generic.ScriptError("Random probability must be higher than 0", self.probability.pos)
-        self.result = action2var.reduce_varaction2_expr(self.result, var_feature)
+        self.result.value = action2var.reduce_varaction2_expr(self.result.value, var_feature)
 
     def debug_print(self, indentation):
-        print indentation*' ' + 'Probability:'
+        generic.print_dbg(indentation, 'Probability:')
         self.probability.debug_print(indentation + 2)
-        print indentation*' ' + 'Result:'
+
+        generic.print_dbg(indentation, 'Result:')
         self.result.debug_print(indentation + 2)
 
     def __str__(self):
-        ret = str(self.probability)
-        if isinstance(self.result, expression.SpriteGroupRef):
-            ret += ': %s;' % str(self.result)
-        else:
-            ret += ': return %s;' % str(self.result)
-        return ret
+        return '{}: {}'.format(self.probability, self.result)
 
