@@ -42,7 +42,7 @@ class Expression(object):
         @param indentation: Indent all printed lines with at least
             C{indentation} spaces.
         """
-        raise NotImplementedError('debug_print must be implemented in expression-subclass %r' % type(self))
+        raise NotImplementedError('debug_print must be implemented in expression-subclass {!r}'.format(type(self)))
 
     def __str__(self):
         """
@@ -50,7 +50,7 @@ class Expression(object):
 
         @return: A string representation of this expression.
         """
-        raise NotImplementedError('__str__ must be implemented in expression-subclass %r' % type(self))
+        raise NotImplementedError('__str__ must be implemented in expression-subclass {!r}'.format(type(self)))
 
     def reduce(self, id_dicts = [], unknown_id_fatal = True):
         """
@@ -63,7 +63,7 @@ class Expression(object):
 
         @return: A deep copy of this expression simplified as much as possible.
         """
-        raise NotImplementedError('reduce must be implemented in expression-subclass %r' % type(self))
+        raise NotImplementedError('reduce must be implemented in expression-subclass {!r}'.format(type(self)))
 
     def reduce_constant(self, id_dicts = []):
         """
@@ -108,7 +108,7 @@ class Expression(object):
         @return: True if the value of this expression is either 0 or 1.
         """
         return False
-    
+
     def type(self):
         """
         Determine the datatype of this expression.
@@ -123,7 +123,7 @@ class ConstantNumeric(Expression):
         self.value = generic.truncate_int32(value)
 
     def debug_print(self, indentation):
-        print indentation*' ' + 'Int:', self.value
+        generic.print_dbg(indentation, 'Int:', self.value)
 
     def write(self, file, size):
         file.print_varx(self.value, size)
@@ -155,13 +155,13 @@ class ConstantFloat(Expression):
         self.value = float(value)
 
     def debug_print(self, indentation):
-        print indentation*' ' + 'Float:', self.value
+        generic.print_dbg(indentation, 'Float:', self.value)
 
     def __str__(self):
         return str(self.value)
 
     def reduce(self, id_dicts = [], unknown_id_fatal = True):
         return self
-    
+
     def type(self):
         return Type.FLOAT
