@@ -14,7 +14,6 @@ with NML; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA."""
 
 import datetime, calendar, math
-from nml.actions import action11
 from nml import generic, nmlop
 from .base_expression import Type, Expression, ConstantNumeric, ConstantFloat
 from .binop import BinOp
@@ -23,7 +22,8 @@ from .parameter import parse_string_to_dword
 from .storage_op import StorageOp
 from .string_literal import StringLiteral
 from .ternaryop import TernaryOp
-import identifier
+from . import identifier
+from functools import reduce
 
 class FunctionCall(Expression):
     def __init__(self, name, params, pos):
@@ -32,37 +32,37 @@ class FunctionCall(Expression):
         self.params = params
 
     def debug_print(self, indentation):
-        print indentation*' ' + 'Call function: ' + self.name.value
+        generic.print_dbg(indentation, 'Call function: ' + self.name.value)
         for param in self.params:
-            print (indentation+2)*' ' + 'Parameter:'
+            generic.print_dbg(indentation + 2, 'Parameter:')
             param.debug_print(indentation + 4)
 
     def __str__(self):
-        ret = ''
-        for param in self.params:
-            if ret == '': ret = str(param)
-            else: ret = '%s, %s' % (ret, str(param))
-        ret = '%s(%s)' % (self.name, ret)
+        ret = '{}({})'.format(self.name, ', '.join(str(param) for param in self.params))
         return ret
 
     def reduce(self, id_dicts = [], unknown_id_fatal = True):
+        # At this point we don't care about invalid arguments, they'll be handled later.
+        identifier.ignore_all_invalid_ids = True
+        params = [param.reduce(id_dicts, unknown_id_fatal = False) for param in self.params]
+        identifier.ignore_all_invalid_ids = False
         if self.name.value in function_table:
             func = function_table[self.name.value]
-            val = func(self.name.value, self.params, self.pos)
+            val = func(self.name.value, params, self.pos)
             return val.reduce(id_dicts)
         else:
             #try user-defined functions
             func_ptr = self.name.reduce(id_dicts, unknown_id_fatal = False, search_func_ptr = True)
             if func_ptr != self.name: # we found something!
                 if func_ptr.type() == Type.SPRITEGROUP_REF:
-                    func_ptr.param_list = self.params
+                    func_ptr.param_list = params
                     return func_ptr
                 if func_ptr.type() != Type.FUNCTION_PTR:
-                    raise generic.ScriptError("'%s' is defined, but it is not a function." % self.name.value, self.pos)
-                return func_ptr.call(self.params)
+                    raise generic.ScriptError("'{}' is defined, but it is not a function.".format(self.name.value), self.pos)
+                return func_ptr.call(params)
             if unknown_id_fatal:
-                raise generic.ScriptError("'%s' is not defined as a function." % self.name.value, self.pos)
-            return FunctionCall(self.name, self.params, self.pos)
+                raise generic.ScriptError("'{}' is not defined as a function.".format(self.name.value), self.pos)
+            return FunctionCall(self.name, params, self.pos)
 
 
 class SpecialCheck(Expression):
@@ -70,7 +70,7 @@ class SpecialCheck(Expression):
     Action7/9 special check (e.g. to see whether a cargo is defined)
 
     @ivar op: Action7/9 operator to use
-    @type op: (C{int}, C{basestring})-tuple
+    @type op: (C{int}, C{str})-tuple
 
     @ivar varnum: Variable number to read
     @type varnum: C{int}
@@ -81,19 +81,23 @@ class SpecialCheck(Expression):
     @ivar value: Value to test
     @type value: C{int}
 
+    @ivar varsize: Varsize for the action7/9 check
+    @type varsize: C{int}
+
     @ivar mask: Mask to to test only certain bits of the value
     @type mask: C{int}
 
     @ivar pos: Position information
     @type pos: L{Position}
     """
-    def __init__(self, op, varnum, results, value, to_string, mask = None, pos = None):
+    def __init__(self, op, varnum, results, value, to_string, varsize = 4, mask = None, pos = None):
         Expression.__init__(self, pos)
         self.op = op
         self.varnum = varnum
         self.results = results
         self.value = value
         self.to_string = to_string
+        self.varsize = varsize;
         self.mask = mask
 
     def reduce(self, id_dicts = [], unknown_id_fatal = True):
@@ -210,7 +214,7 @@ def builtin_day_of_year(name, args, pos):
     # Mapping of month to number of days in that month.
     number_days = {1: 31, 2: 28, 3: 31, 4: 30, 5: 31, 6: 30, 7: 31, 8: 31, 9: 30, 10: 31, 11: 30, 12: 31}
     if day.value < 1 or day.value > number_days[month.value]:
-        raise generic.ScriptError('Day should be value between 1 and %d.' % number_days[month.value], day.pos)
+        raise generic.ScriptError('Day should be value between 1 and {:d}.'.format(number_days[month.value]), day.pos)
 
     return ConstantNumeric(datetime.date(1, month.value, day.value).toordinal(), pos)
 
@@ -245,6 +249,22 @@ def builtin_hasbit(name, args, pos):
         raise generic.ScriptError(name + "() must have exactly two parameters", pos)
     return BinOp(nmlop.HASBIT, args[0], args[1], pos)
 
+def builtin_getbits(name, args, pos):
+    """
+    getbits(value, first, amount) builtin function.
+
+    @return Extract C{amount} bits starting at C{first} from C{value}, that is (C{value} >> C{first}) & (1 << C{amount} - 1)
+    """
+    if len(args) != 3:
+        raise generic.ScriptError(name + "() must have exactly three parameters", pos)
+
+    # getbits(value, first, amount) = (value >> first) & ((0xFFFFFFFF << amount) ^ 0xFFFFFFFF)
+    part1 = BinOp(nmlop.SHIFTU_RIGHT, args[0], args[1], pos)
+    part2 = BinOp(nmlop.SHIFT_LEFT, ConstantNumeric(0xFFFFFFFF), args[2], pos)
+    part3 = BinOp(nmlop.XOR, part2, ConstantNumeric(0xFFFFFFFF), pos)
+
+    return BinOp(nmlop.AND, part1, part3, pos)
+
 def builtin_version_openttd(name, args, pos):
     """
     version_openttd(major, minor, revision[, build]) builtin function.
@@ -268,7 +288,7 @@ def builtin_cargotype_available(name, args, pos):
     if len(args) != 1:
         raise generic.ScriptError(name + "() must have exactly 1 parameter", pos)
     label = args[0].reduce()
-    return SpecialCheck((0x0B, r'\7c'), 0, (0, 1), parse_string_to_dword(label), "%s(%s)" % (name, str(label)), pos = args[0].pos)
+    return SpecialCheck((0x0B, r'\7c'), 0, (0, 1), parse_string_to_dword(label), "{}({})".format(name, str(label)), pos = args[0].pos)
 
 def builtin_railtype_available(name, args, pos):
     """
@@ -279,7 +299,7 @@ def builtin_railtype_available(name, args, pos):
     if len(args) != 1:
         raise generic.ScriptError(name + "() must have exactly 1 parameter", pos)
     label = args[0].reduce()
-    return SpecialCheck((0x0D, None), 0, (0, 1), parse_string_to_dword(label), "%s(%s)" % (name, str(label)), pos = args[0].pos)
+    return SpecialCheck((0x0D, None), 0, (0, 1), parse_string_to_dword(label), "{}({})".format(name, str(label)), pos = args[0].pos)
 
 def builtin_grf_status(name, args, pos):
     """
@@ -303,10 +323,12 @@ def builtin_grf_status(name, args, pos):
     else:
         assert False, "Unknown grf status function"
     if mask is None:
-        string = "%s(%s)" % (name, str(label))
+        string = "{}({})".format(name, str(labels))
+        varsize = 4
     else:
-        string = "%s(%s, %s)" % (name, str(label), str(mask))
-    return SpecialCheck(op, 0x88, results, parse_string_to_dword(labels[0]), string, mask, args[0].pos)
+        string = "{}({}, {})".format(name, str(labels), str(mask))
+        varsize = 8
+    return SpecialCheck(op, 0x88, results, parse_string_to_dword(labels[0]), string, varsize, mask, args[0].pos)
 
 def builtin_visual_effect_and_powered(name, args, pos):
     """
@@ -315,11 +337,11 @@ def builtin_visual_effect_and_powered(name, args, pos):
     visual_effect(effect, offset)
     Use this to set the vehicle property visual_effect[_and_powered]
     and for the callback VEH_CB_VISUAL_EFFECT[_AND_POWERED]
-    
+
     """
     arg_len = 2 if name == 'visual_effect' else 3
     if len(args) != arg_len:
-        raise generic.ScriptError(name + "() must have %d parameters" % arg_len, pos)
+        raise generic.ScriptError(name + "() must have {:d} parameters".format(arg_len), pos)
     from nml import global_constants
     effect = args[0].reduce_constant(global_constants.const_list).value
     offset = BinOp(nmlop.ADD, args[1], ConstantNumeric(8), args[1].pos).reduce_constant().value
@@ -332,6 +354,30 @@ def builtin_visual_effect_and_powered(name, args, pos):
         powered = 0
     return ConstantNumeric(effect | offset | powered)
 
+def builtin_create_effect(name, args, pos):
+    """
+    Builtin function:
+    create_effect(effect_sprite, l_x_offset, t_y_offset, z_offset)
+    Use this to set the values for temporary storages 100+x
+    in the callback create_effect
+
+    """
+    if len(args) != 4:
+        raise generic.ScriptError(name + "() must have 4 parameters", pos)
+    from nml import global_constants
+
+    sprite = args[0].reduce_constant(global_constants.const_list).value
+    offset1 = args[1].reduce_constant().value
+    offset2 = args[2].reduce_constant().value
+    offset3 = args[3].reduce_constant().value
+
+    generic.check_range(sprite, 0, 255, "effect_sprite in function " + name, args[0].pos)
+    generic.check_range(offset1, -128, 127, "l_x_offset in function " + name, args[1].pos)
+    generic.check_range(offset2, -128, 127, "t_y_offset in function " + name, args[2].pos)
+    generic.check_range(offset3, -128, 127, "z_offset in function " + name, args[3].pos)
+
+    return ConstantNumeric(sprite | (offset1 & 0xFF) << 8 | (offset2 & 0xFF) << 16 | (offset3 & 0xFF) << 24)
+
 def builtin_str2number(name, args, pos):
     if len(args) != 1:
         raise generic.ScriptError(name + "() must have 1 parameter", pos)
@@ -357,7 +403,7 @@ def builtin_reserve_sprites(name, args, pos):
     if len(args) != 1:
         raise generic.ScriptError(name + "() must have 1 parameter", pos)
     count = args[0].reduce_constant()
-    func = lambda x: '%s(%d)' % (name, count.value)
+    func = lambda x: '{}({:d})'.format(name, count.value)
     return GRMOp(nmlop.GRM_RESERVE, 0x08, count.value, func, pos)
 
 def builtin_industry_type(name, args, pos):
@@ -374,10 +420,10 @@ def builtin_industry_type(name, args, pos):
     if type not in (0, 1):
         raise generic.ScriptError("First argument of industry_type() must be IND_TYPE_OLD or IND_TYPE_NEW", pos)
 
-    # Industry ID uses 6 bits (0 .. 5), so bit 6 is never used
+    # Industry ID uses 7 bits (0 .. 6), bit 7 is for old/new
     id = args[1].reduce_constant(global_constants.const_list).value
-    if not 0 <= id <= 63:
-        raise generic.ScriptError("Second argument 'id' of industry_type() must be in range 0..63", pos)
+    if not 0 <= id <= 127:
+        raise generic.ScriptError("Second argument 'id' of industry_type() must be in range 0..127", pos)
 
     return ConstantNumeric(type << 7 | id)
 
@@ -412,18 +458,24 @@ def builtin_abs(name, args, pos):
     return TernaryOp(guard, BinOp(nmlop.SUB, ConstantNumeric(0), args[0], args[0].pos), args[0], args[0].pos).reduce()
 
 def builtin_sound_file(name, args, pos):
-    if len(args) != 1:
-        raise generic.ScriptError(name + "() must have 1 parameter", pos)
+    from nml.actions import action11
+    if len(args) not in (1, 2):
+        raise generic.ScriptError(name + "() must have 1 or 2 parameters", pos)
     if not isinstance(args[0], StringLiteral):
         raise generic.ScriptError("Parameter for " + name + "() must be a string literal", pos)
-    return ConstantNumeric(action11.add_sound(args[0].value), pos)
+    volume = args[1].reduce_constant().value if len(args) >= 2 else 100
+    generic.check_range(volume, 0, 100, "sound volume", pos)
+    return ConstantNumeric(action11.add_sound( (args[0].value, volume), pos), pos)
 
 def builtin_sound_import(name, args, pos):
-    if len(args) != 2:
-        raise generic.ScriptError(name + "() must have 2 parameters", pos)
+    from nml.actions import action11
+    if len(args) not in (2, 3):
+        raise generic.ScriptError(name + "() must have 2 or 3 parameters", pos)
     grfid = parse_string_to_dword(args[0].reduce())
     sound_num = args[1].reduce_constant().value
-    return ConstantNumeric(action11.add_sound((grfid, sound_num)), pos)
+    volume = args[2].reduce_constant().value if len(args) >= 3 else 100
+    generic.check_range(volume, 0, 100, "sound volume", pos)
+    return ConstantNumeric(action11.add_sound( (grfid, sound_num, volume), pos), pos)
 
 def builtin_relative_coord(name, args, pos):
     """
@@ -435,9 +487,9 @@ def builtin_relative_coord(name, args, pos):
         raise generic.ScriptError(name + "() must have x and y coordinates as parameters", pos)
 
     if isinstance(args[0], ConstantNumeric):
-        generic.check_range(args[0].value, 0, 255, "Argument of '%s'" % name, args[0].pos)
+        generic.check_range(args[0].value, 0, 255, "Argument of '{}'".format(name), args[0].pos)
     if isinstance(args[1], ConstantNumeric):
-        generic.check_range(args[1].value, 0, 255, "Argument of '%s'" % name, args[1].pos)
+        generic.check_range(args[1].value, 0, 255, "Argument of '{}'".format(name), args[1].pos)
 
     x_coord = BinOp(nmlop.AND, args[0], ConstantNumeric(0xFF), args[0].pos)
     y_coord = BinOp(nmlop.AND, args[1], ConstantNumeric(0xFF), args[1].pos)
@@ -480,7 +532,7 @@ def builtin_slope_to_sprite_offset(name, args, pos):
         raise generic.ScriptError(name + "() must have 1 parameter", pos)
 
     if isinstance(args[0], ConstantNumeric):
-        generic.check_range(args[0].value, 0, 15, "Argument of '%s'" % name, args[0].pos)
+        generic.check_range(args[0].value, 0, 15, "Argument of '{}'".format(name), args[0].pos)
 
     # step 1: ((slope >= 0) & (slope <= 14)) * slope
     # This handles all non-steep slopes
@@ -505,7 +557,7 @@ def builtin_palette_1cc(name, args, pos):
         raise generic.ScriptError(name + "() must have 1 parameter", pos)
 
     if isinstance(args[0], ConstantNumeric):
-        generic.check_range(args[0].value, 0, 15, "Argument of '%s'" % name, args[0].pos)
+        generic.check_range(args[0].value, 0, 15, "Argument of '{}'".format(name), args[0].pos)
 
     return BinOp(nmlop.ADD, args[0], ConstantNumeric(775), pos)
 
@@ -520,7 +572,7 @@ def builtin_palette_2cc(name, args, pos):
 
     for i in range(0, 2):
         if isinstance(args[i], ConstantNumeric):
-            generic.check_range(args[i].value, 0, 15, "Argument of '%s'" % name, args[i].pos)
+            generic.check_range(args[i].value, 0, 15, "Argument of '{}'".format(name), args[i].pos)
 
     col2 = BinOp(nmlop.MUL, args[1], ConstantNumeric(16), pos)
     col12 = BinOp(nmlop.ADD, col2, args[0], pos)
@@ -541,12 +593,39 @@ def builtin_vehicle_curv_info(name, args, pos):
 
     for arg in args:
         if isinstance(arg, ConstantNumeric):
-            generic.check_range(arg.value, -2, 2, "Argument of '%s'" % name, arg.pos)
+            generic.check_range(arg.value, -2, 2, "Argument of '{}'".format(name), arg.pos)
 
     args = [BinOp(nmlop.AND, arg, ConstantNumeric(0xF), pos) for arg in args]
     cur_next = BinOp(nmlop.SHIFT_LEFT, args[1], ConstantNumeric(8), pos)
     return BinOp(nmlop.OR, args[0], cur_next, pos)
 
+def builtin_format_string(name, args, pos):
+    """
+    format_string(format, ... args ..) builtin function
+
+    @return Formatted string
+    """
+    if len(args) < 1:
+        raise generic.ScriptError(name + "() must have at least one parameter", pos)
+
+    format = args[0].reduce()
+    if not isinstance(format, StringLiteral):
+        raise generic.ScriptError(name + "() parameter 1 'format' must be a literal string", format.pos)
+
+    # Validate other args
+    format_args = []
+    for i, arg in enumerate(args[1:]):
+        arg = arg.reduce()
+        if not isinstance(arg, (StringLiteral, ConstantFloat, ConstantNumeric)):
+            raise generic.ScriptError(name + "() parameter {:d} is not a constant number of literal string".format(i+1), arg.pos)
+        format_args.append(arg.value)
+
+    try:
+        result = format.value % tuple(format_args)
+        return StringLiteral(result, pos)
+    except Exception as ex:
+        raise generic.ScriptError("Invalid combination of format / arguments for {}: {}".format(name, str(ex)), pos)
+
 #}
 
 function_table = {
@@ -560,6 +639,7 @@ function_table = {
     'LOAD_TEMP' : builtin_storage,
     'LOAD_PERM' : builtin_storage,
     'hasbit' : builtin_hasbit,
+    'getbits' : builtin_getbits,
     'version_openttd' : builtin_version_openttd,
     'cargotype_available' : builtin_cargotype_available,
     'railtype_available' : builtin_railtype_available,
@@ -568,6 +648,7 @@ function_table = {
     'grf_order_behind' : builtin_grf_status,
     'visual_effect' : builtin_visual_effect_and_powered,
     'visual_effect_and_powered' : builtin_visual_effect_and_powered,
+    'create_effect' : builtin_create_effect,
     'str2number' : builtin_str2number,
     'cargotype' : builtin_cargotype,
     'railtype' : builtin_railtype,
@@ -592,4 +673,5 @@ function_table = {
     'palette_1cc' : builtin_palette_1cc,
     'palette_2cc' : builtin_palette_2cc,
     'vehicle_curv_info' : builtin_vehicle_curv_info,
+    'format_string' : builtin_format_string,
 }
