@@ -16,7 +16,6 @@ with NML; if not, write to the Free Software Foundation, Inc.,
 from nml import generic
 from .base_expression import ConstantNumeric, Expression, Type
 from .parameter import parse_string_to_dword
-from .string_literal import StringLiteral
 
 storage_op_info = {
     'STORE_PERM' : {'store': True,  'perm': True,  'grfid': False, 'max': 0x0F},
@@ -30,7 +29,7 @@ class StorageOp(Expression):
     Class for reading/writing to (temporary or permanent) storage
 
     @ivar name: Name of the called storage function
-    @type name: C{str} or C{unicode}
+    @type name: C{str}
 
     @ivar info: Dictionary containting information about the operation to perform
     @type info: C{dict}
@@ -54,8 +53,8 @@ class StorageOp(Expression):
         arg_len = (2,) if self.info['store'] else (1,)
         if self.info['grfid']: arg_len += (arg_len[0] + 1,)
         if len(args) not in arg_len:
-            argstr = "%d" % arg_len[0] if len(arg_len) == 1 else "%d..%d" % arg_len
-            raise generic.ScriptError("%s requires %s argument(s), encountered %d" % (name, argstr, len(args)), pos)
+            argstr = "{:d}".format(arg_len[0]) if len(arg_len) == 1 else "{}..{}".format(arg_len[0], arg_len[1])
+            raise generic.ScriptError("{} requires {} argument(s), encountered {:d}".format(name, argstr, len(args)), pos)
 
         i = 0
         if self.info['store']:
@@ -75,14 +74,14 @@ class StorageOp(Expression):
 
 
     def debug_print(self, indentation):
-        print indentation*' ' + self.name
-        print (indentation+2)*' ' + 'Register:'
+        generic.print_dbg(indentation, self.name)
+        generic.print_dbg(indentation + 2, 'Register:')
         self.register.debug_print(indentation + 4)
         if self.value is not None:
-            print (indentation+2)*' ' + 'Value:'
+            generic.print_dbg(indentation + 2, 'Value:')
             self.value.debug_print(indentation + 4)
         if self.grfid is not None:
-            print (indentation+2)*' ' + 'GRFID:'
+            generic.print_dbg(indentation + 2, 'GRFID:')
             self.grfid.debug_print(indentation + 4)
 
     def __str__(self):
@@ -90,7 +89,7 @@ class StorageOp(Expression):
         if self.value is not None: args.append(str(self.value))
         args.append(str(self.register))
         if self.grfid is not None: args.append(str(self.grfid))
-        return "%s(%s)" % (self.name, ", ".join(args))
+        return "{}({})".format(self.name, ", ".join(args))
 
     def reduce(self, id_dicts = [], unknown_id_fatal = True):
         args = []
@@ -104,7 +103,7 @@ class StorageOp(Expression):
         if register.type() != Type.INTEGER:
             raise generic.ScriptError("Register to access must be an integer.", register.pos)
         if isinstance(register, ConstantNumeric) and register.value > self.info['max']:
-            raise generic.ScriptError("Maximum register for %s is %d" % (self.name, self.info['max']), self.pos)
+            raise generic.ScriptError("Maximum register for {} is {:d}".format(self.name, self.info['max']), self.pos)
         args.append(register)
 
         if self.grfid is not None:
@@ -120,5 +119,5 @@ class StorageOp(Expression):
 
     def supported_by_actionD(self, raise_error):
         if raise_error:
-            raise generic.ScriptError("%s() may only be used inside switch-blocks" % self.name, self.pos)
+            raise generic.ScriptError("{}() may only be used inside switch-blocks".format(self.name), self.pos)
         return False
