@@ -13,21 +13,28 @@ You should have received a copy of the GNU General Public License along
 with NML; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA."""
 
+from nml import generic
 from .base_expression import Type, Expression
 
 class StringLiteral(Expression):
+    """
+    String literal expression.
+
+    @ivar value: Value of the string literal.
+    @type value: C{str}
+    """
     def __init__(self, value, pos):
         Expression.__init__(self, pos)
         self.value = value
 
     def debug_print(self, indentation):
-        print indentation*' ' + 'String literal: "%s"' % self.value
+        generic.print_dbg(indentation, 'String literal: "{}"'.format(self.value))
 
     def __str__(self):
-        return '"%s"' % self.value
+        return '"{}"'.format(self.value)
 
     def write(self, file, size):
-        assert(len(self.value) == size)
+        assert len(self.value) == size
         file.print_string(self.value, final_zero = False, force_ascii = True)
 
     def reduce(self, id_dicts = [], unknown_id_fatal = True):
