@@ -207,6 +207,7 @@ commands = {
 'CARGO_LONG':     {'unicode': r'\UE09A\1B', 'ascii': r'\9A\1B', 'size': 2 * 2},
 'CARGO_SHORT':    {'unicode': r'\UE09A\1C', 'ascii': r'\9A\1C', 'size': 2 * 2},
 'CARGO_TINY':     {'unicode': r'\UE09A\1D', 'ascii': r'\9A\1D', 'size': 2 * 2},
+'CARGO_NAME':     {'unicode': r'\UE09A\1E', 'ascii': r'\9A\1E', 'size': 2},
 
 # Colors
 'BLUE':           {'unicode': r'\UE088',    'ascii': r'\88'},
@@ -943,7 +944,7 @@ class Language(object):
             parsed_string += SET_STRING_GENDER[str_type] + '\\{:02X}'.format(self.genders[self.strings[string_id].gender])
         if len(self.strings[string_id].cases) > 0:
             parsed_string += BEGIN_CASE_CHOICE_LIST[str_type]
-            for case_name, case_string in list(self.strings[string_id].cases.items()):
+            for case_name, case_string in sorted(self.strings[string_id].cases.items()):
                 case_id = self.cases[case_name]
                 parsed_string += CHOICE_LIST_ITEM[str_type] + '\\{:02X}'.format(case_id) + case_string.parse_string(str_type, self, lang_id, string.params)
             parsed_string += CHOICE_LIST_DEFAULT[str_type]
@@ -967,7 +968,7 @@ class Language(object):
             try:
                 value = int(lang_text, 16)
             except ValueError:
-                raise generic.ScriptError("Invalid grflangid {:r}".format(lang_text), pos)
+                raise generic.ScriptError("Invalid grflangid {!r}".format(lang_text), pos)
         if value < 0 or value >= 0x7F:
             raise generic.ScriptError("Invalid grflangid", pos)
         self.langid = value
