@@ -18,37 +18,66 @@ with NML; if not, write to the Free Software Foundation, Inc.,
 # get_numeric_version is used only for the purpose of packet creation,
 # in all other cases use get_nml_version()
 
-import subprocess, os, sys
+import subprocess, os
+
+def get_child_output(cmd):
+    """
+    Run a child process, and collect the generated output.
+
+    @param cmd: Command to execute.
+    @type  cmd: C{list} of C{str}
+
+    @return: Generated output of the command, split on whitespace.
+    @rtype:  C{list} of C{str}
+    """
+    return subprocess.check_output(cmd, universal_newlines = True).split()
+
 
 def get_hg_version():
-    path = os.path.dirname(os.path.realpath(sys.argv[0]))
+    path = os.path.dirname(os.path.dirname(os.path.realpath(__file__)))
     version = ''
     if os.path.isdir(os.path.join(path,'.hg')):
         # we want to return to where we were. So save the old path
-        version_list = (subprocess.Popen(['hg', '-R', path, 'id', '-n', '-t', '-i'], stdout=subprocess.PIPE).communicate()[0]).split()
+        try:
+            version_list = get_child_output(['hg', '-R', path, 'id', '-n', '-t', '-i'])
+        except OSError as e:
+            print("Mercurial checkout found but cannot determine its version. Error({0}): {1}".format(e.errno, e.strerror))
+            return version
+
         if version_list[1].endswith('+'):
             modified = 'M'
         else:
             modified = ''
-        revision = version_list[1].rstrip('+')
-        hash = version_list[0].rstrip('+')
-        # Test whether we have a tag (=release version)
-        if len(version_list) > 2 and version_list[2] != 'tip':
+
+        # Test whether we have a tag (=release version) and add it, if found
+        if len(version_list) > 2 and version_list[2] != 'tip' and modified == '':
+            # Tagged release
             version = version_list[2]
-        else: # we have a trunk version
-            version = 'r'+revision
-        # Add modification tag and hash to the stored version info
-        version += modified + ' (' + hash + ')'
+        else:
+            # Branch or modified version
+            hash = version_list[0].rstrip('+')
+
+            # Get the date of the commit of the current NML version in days since January 1st 2000
+            ctimes = get_child_output(["hg", "-R", path, "parent", "--template='{date|hgdate} {date|shortdate}\n'"])
+            ctime = (int((ctimes[0].split("'"))[1]) - 946684800) // (60 * 60 * 24)
+            cversion = str(ctime)
+
+            # Combine the version string
+            version = "v{}{}:{} from {}".format(cversion, modified, hash, ctimes[2].split("'", 1)[0])
     return version
 
 def get_lib_versions():
     versions = {}
     #PIL
     try:
-        import Image
+        from PIL import Image
         versions["PIL"] = Image.VERSION
     except ImportError:
-        versions["PIL"] = "Not found!"
+        try:
+            import Image
+            versions["PIL"] = Image.VERSION
+        except ImportError:
+            versions["PIL"] = "Not found!"
 
     #PLY
     try:
@@ -76,7 +105,7 @@ def get_cli_version():
     #version string for usage in command line
     result = get_nml_version() + "\n"
     result += "Library versions encountered:\n"
-    for lib, lib_ver in get_lib_versions().items():
+    for lib, lib_ver in list(get_lib_versions().items()):
         result += lib + ": " + lib_ver + "\n"
     return result[0:-1] #strip trailing newline
 
@@ -84,11 +113,11 @@ def get_and_write_version():
     version = get_nml_version()
     if version:
         try:
-            path = os.path.dirname(os.path.realpath(sys.argv[0]))
-            f = open(os.path.join(path,"nml/__version__.py"), "w")
+            path = os.path.dirname(os.path.dirname(os.path.realpath(__file__)))
+            f = open(os.path.join(path, "nml", "__version__.py"), "w")
             f.write('# this file is autogenerated by setup.py\n')
-            f.write('version = "%s"\n' % version)
+            f.write('version = "{}"\n'.format(version))
             f.close()
-            return (get_nml_version().split())[0]
+            return version.split()[0]
         except IOError:
-            print "Version file NOT written"
+            print("Version file NOT written")
